/* ----------------------------------------------------------------- */
/*  Copyright 1999 IBM Corporation.  All rights reserved.            */
/* ----------------------------------------------------------------- */

/**
 * File:             ServletGroupBean.java
 * Date of Creation: 03/01/1999
 *   
 * Description: ServletGroup Admin Repository Object Bean implementation.
 *       
 * @author Michael J. Morton
 * @88536 delta from cmvc asv302 1.21
 */ 
package com.ibm.ejs.sm.beans;

import java.util.*;
import java.sql.*;
import java.rmi.RemoteException;
import javax.ejb.*;
import java.io.*;
import com.ibm.ejs.oa.*;
import com.ibm.ejs.ras.*;
import com.ibm.ejs.util.jar.*;
import com.ibm.ejs.util.*;
import com.ibm.ejs.sm.exception.*;
import com.ibm.ejs.sm.active.ActiveObjectConfig;
import com.ibm.ejs.sm.active.ActiveObjectStatus;
import com.ibm.ejs.sm.active.ActiveServlet;
import com.ibm.ejs.sm.active.ActiveServletGroup;
import com.ibm.ejs.sm.active.ActiveServletGroupConfig;
import com.ibm.ejs.sm.agent.AdminAgent;
import com.ibm.ejs.sm.agent.ParamList;
import com.ibm.ejs.sm.agent.ContainmentPath;
import com.ibm.ejs.sm.agent.ContainmentPathElem;
import com.ibm.ejs.sm.client.ui.NLS;
import com.ibm.ejs.sm.util.act.Act;
import com.ibm.ejs.sm.util.act.ActServerImpl;
import com.ibm.ejs.sm.util.db.DBMgr;
import com.ibm.ejs.sm.util.db.DBQueryResult;
import com.ibm.ejs.sm.util.Utils;
import com.ibm.servlet.engine.oselistener.systemsmgmt.PluginCfgGenerator;

public class ServletGroupBean extends LiveRepositoryObjectImpl
implements EntityBean
{
	private static final String PRODUCT_PROPS_PATH = "/com/ibm/websphere/product.properties";

	// Lock for synchronizing static variables.
	private static Integer classLock = new Integer(0);

	// Constant for Trace Component for this class.
	private static final TraceComponent tc = Tr.register(ServletGroupBean.class);

	// Class/Constants for initialization of this class.
	private static NLS nls = null;
	protected static final String myInterfaceName = RepositoryObjectImpl.qualifyClassName("ServletGroup");
	private static final String myClassName = RepositoryObjectImpl.qualifyClassName("ServletGroupBean");
	private static boolean tableCreated = false;
	private static Type myTypeObj = null;
	private static Long myTypeId = null;

	// Instance variables.
	private boolean dirty;
	private boolean hasContext = false;
	private ServletEngine initialServletEngine = null;

	// Instance variables for ServletGroup attributes.
	private String name;
	private String description;
	private String docRoot;
	private Vector classpath;
	private boolean sharedContext;
	private String sharedContextJNDIName;
	private String errorPage;
	private Properties filterList;
	private Properties groupAttributes;
	private boolean autoReload;
	private int reloadInterval;
	private boolean enabled;
	private String groupURI;

	/*
	 * Binary attribute names.
	 */
	private static final String classpathBinAttrName = myInterfaceName +
																										 ".classpath";
	private static final String filterListBinAttrName = myInterfaceName +
																											".filterList";
	private static final String groupAttributesBinAttrName = myInterfaceName +
																													 ".groupAttributes";

	// Constants for default ServletGroup attribute values.
	private static final String defaultDescription = new String();
	private static final String defaultDocRoot = new String();
	private static final Vector defaultClasspath = new Vector();
	private static final boolean defaultSharedContext = false;
	private static final String defaultSharedContextJNDIName = "SrdSrvltCtxHome";
	private static final String defaultErrorPage = new String();
	private static final Properties defaultFilterList = new Properties();
	private static final Properties defaultGroupAttrs = new Properties();
	private static final boolean defaultAutoReload = true;
	private static final int defaultReloadInterval = 3000;
	private static final boolean defaultEnabled = true;
	private static final String defaultGroupURI = new String();

	// Constants for location of attributes in ServletGroup Admin Table.
	private static final String tableName = DBMgr.qualifiedTableName("SRVLT_GRP_TBL");
	private static final String descriptionColumnName = "Description";
	private static final int descriptionColumnIndex = columnIndexBase + 1;
	private static final String docRootColumnName = "DocRoot";
	private static final int docRootColumnIndex = columnIndexBase + 2;
	private static final String sharedContextColumnName = "SharedContext";
	private static final int sharedContextColumnIndex = columnIndexBase + 3;
	private static final String sharedContextJNDINameColumnName = "SrdCtxJNDIName";
	private static final int sharedContextJNDINameColumnIndex = columnIndexBase + 4;
	private static final String errorPageColumnName = "ErrorPage";
	private static final int errorPageColumnIndex = columnIndexBase + 5;
	private static final String autoReloadColumnName = "AutoReload";
	private static final int autoReloadColumnIndex = columnIndexBase + 6;
	private static final String reloadIntervalColumnName = "ReloadInterval";
	private static final int reloadIntervalColumnIndex = columnIndexBase + 7;
	private static final String enabledColumnName = "Enabled";
	private static final int enabledColumnIndex = columnIndexBase + 8;
	private static final String groupURIColumnName = "GroupURI";
	private static final int groupURIColumnIndex = columnIndexBase + 9;
	private static final int numColumns = columnIndexBase + 9;

	// Constants for index keys into prepared statement cache.
	private static final int groupBeanKeyBase = DBMgr.getPreparedStmtCacheKeyBase();
	private static final int loadStmtKey = groupBeanKeyBase + 1;
	private static final int deleteStmtKey = groupBeanKeyBase + 2;
	private static final int updateStmtKey = groupBeanKeyBase + 3;
	private static final int findAllStmtKey = groupBeanKeyBase + 4;
	private static final int restrictedFindAllStmtKey = groupBeanKeyBase + 5;
	private static final int insertStmtKey = groupBeanKeyBase + 6;

	// Constants for Sql strings for manipulating the ServletGroup Admin Table.
	private static final String insertStmtSql = "insert into " + 
																							tableName +
																							" values (?,?,?,?,?,?,?,?,?,?,?)";

	private static final String updateStmtSql = "update " + 
																							tableName + 
																							" set " +
																							descriptionColumnName +
																							" = ? ," +  
																							docRootColumnName +
																							" = ? ," +  
																							sharedContextColumnName +
																							" = ? ," +
																							sharedContextJNDINameColumnName +
																							" = ? ," +
																							errorPageColumnName +
																							" = ? ," +
																							autoReloadColumnName +
																							" = ? ," +
																							reloadIntervalColumnName +
																							" = ? ," +
																							enabledColumnName +
																							" = ? ," +
																							groupURIColumnName +
																							" = ? " +
																							" where " + 
																							instanceIdColumnName + 
																							" = ?"; 

	private static final String deleteStmtSql = "delete from " + 
																							tableName + 
																							" where " +
																							instanceIdColumnName +  
																							" =  ?";

	// Static variables for information about relationships in which this
	// bean participates.
	private static Relation engineGroupRel;
	protected static final String servletEngineGroupRelName = "servletEngineGroupRel";  
	private static Relation groupServletRel;
	protected static final String groupServletRelName = "groupServletRel";  

	// Constant for active object class name corresponding to this bean.
	private static final String activeObjectClassName = "ActiveServletGroup";


	private static final String applicationServletGroupRelName = "applicationServletGroupRelation";
	private static Relation applicationServletGroupRel;

	// Instance variables for related objects.
	private EJBObject containingObject;
	private ContainmentPathElem activeObjectName;
	private VirtualHost virtualHost;


	public ServletGroupBean() 
	throws RemoteException 
	{
		super();

		// Initialize for NLS.
		initializeNLS("AppServerROStrings");

		initializePersistentStore();
		initializeTypeId();
		// Lookup the servletgroup-servlet relationship to make sure it exits.
		lookupGroupServletRel();
		lookupApplicationServletGroupRel();
	}

	/*
	 * getAttributes -- Return the Attributes for this instance.
	 */
	public Attributes getAttributes(Attributes attributeNames)
	throws RemoteException, OpException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getAttributes");

		// Check to see if this instance is a sub-type of the ServletGroup bean.
		if (narrowRef != null)
		{
			if (tc.isEntryEnabled()) Tr.exit(tc, "getAttributes");
			return((ServletGroup)narrowRef).getAttributes(attributeNames);
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "getAttributes");
		return readInstanceVariables(attributeNames);
	}

	/*
	 * setAttributes - Set the Attributes for this instance.
	 */
	public void setAttributes(Attributes genericAttrs)
	throws RemoteException, OpException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "setAttributes");

		if (narrowRef != null)
		{
			((ServletGroup)narrowRef).setAttributes(genericAttrs);
			if (tc.isEntryEnabled()) Tr.exit(tc, "setAttributes");
			return;
		}

		//mjm-55032
		renameIfNec(genericAttrs, lookupEngineGroupRel());

		try
		{
			Model model = isAClone();
			boolean isCloneInstance = false;

			if (model != null)
			{
				isCloneInstance = true;
			}
			updateInstanceVariables((ServletGroupAttributes)genericAttrs, false, isCloneInstance);
		}
		catch (InvalidURINameException e)
		{
			RemoteException r = new RemoteException("", e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "setAttributes");
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "setAttributes");
	}

	/*
	 * getActiveAttributes -- Get ServletGroup Active Attributes.
	 */
	private ServletGroupAttributes getActiveAttributes(Attributes attributeNames) 
	throws RemoteException, OpException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getActiveAttributes");

		ServletGroupAttributes groupAttrs = (ServletGroupAttributes) attributeNames;
		ActiveServletGroupConfig config = null;

		if (getDesiredState() == ActiveObjectStatus.RUNNING)
		{
			try
			{
				config = (ActiveServletGroupConfig) getRuntimeAttr("Config");
			}
			catch (Exception e)
			{
				if (tc.isEntryEnabled()) Tr.event(tc, "Exception occurred fetching config: ", e);
				config = null;
			}
		}

		if (config == null)
		{
			config = (ActiveServletGroupConfig) getConfig(); 
		}

		groupAttrs.setDescriptionActive(config.getDescription());
		groupAttrs.setDocRootActive(config.getDocRoot());
		groupAttrs.setClasspathActive(config.getClasspath());
		groupAttrs.setSharedContextActive(config.getSharedContext());
		groupAttrs.setSharedContextJNDINameActive(config.getSharedContextJNDIName());
		groupAttrs.setErrorPageActive(config.getErrorPage());
		groupAttrs.setFilterListActive(config.getFilterList());
		groupAttrs.setGroupAttributesActive(config.getGroupAttributes());
		groupAttrs.setAutoReloadActive(config.getAutoReload());
		groupAttrs.setReloadIntervalActive(config.getReloadInterval());
		groupAttrs.setEnabledActive(config.getEnabled());
		groupAttrs.setURIPathActive(config.getURIPath());

		if (tc.isEntryEnabled()) Tr.exit(tc, "getActiveAttributes");

		return groupAttrs;
	}

	/*
	 * intializePersistentStore -- Create ServletGroup table in DB if necessary.
	 */
	private static void initializePersistentStore() 
	throws RemoteException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "initializePersistentStore");

		synchronized (classLock)
		{
			if (tableCreated)
			{
				if (tc.isEntryEnabled()) Tr.exit(tc, "initializePersistentStore");
				return;
			}
		}

		try
		{
			String createTableSql =  "create table " + tableName + " ( " + 
															 instanceIdColumnName + DBMgr.idColumnSpec + " , " +  
															 typeIdColumnName + DBMgr.idColumnSpec + " , " + 
															 descriptionColumnName + DBMgr.nullableMediumStringColumnSpec + " , " + 
															 docRootColumnName + DBMgr.nullableBigStringColumnSpec + " , " + 
															 sharedContextColumnName + " INTEGER NOT NULL , " +
															 sharedContextJNDINameColumnName + DBMgr.nullableMediumStringColumnSpec + " , " +
															 errorPageColumnName + DBMgr.nullableMediumStringColumnSpec + " , " +
															 autoReloadColumnName + " INTEGER NOT NULL , " +
															 reloadIntervalColumnName + " INTEGER NOT NULL , " +
															 enabledColumnName + " INTEGER NOT NULL , " +
															 groupURIColumnName + DBMgr.bigStringColumnSpec + " , " +
															 " primary key(" +  instanceIdColumnName + "))";

			DBMgr.createTableIfNonExistent(tableName, createTableSql);

			synchronized (classLock)
			{
				tableCreated = true;
			}
		}
		catch (SQLException s)
		{
			RemoteException r = new RemoteException((nls.getString("segb.db.create.exception",
																														 "Database create table failed.")),
																							s);
			if (tc.isEntryEnabled()) Tr.exit(tc, "initializePersistentStore", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "initializePersistentStore");
	}

	/*
	 * intializeTypeId -- Registration with the type bean to obtain a 
	 *                    type identifier.
	 */
	private static void initializeTypeId() 
	throws RemoteException 
	{ 
		if (tc.isEntryEnabled()) Tr.entry(tc, "initializeTypeId");

		synchronized (classLock) 
		{
			if (myTypeId != null)
			{
				if (tc.isEntryEnabled()) Tr.exit(tc, "initializeTypeId");
				return;
			}
		}

		ServletGroupAttributes defaultAttrs = new ServletGroupAttributes();

		defaultAttrs.setDescription(defaultDescription);
		defaultAttrs.setDocRoot(defaultDocRoot);
		defaultAttrs.setClasspath(defaultClasspath);
		defaultAttrs.setSharedContext(defaultSharedContext);
		defaultAttrs.setSharedContextJNDIName(defaultSharedContextJNDIName);
		defaultAttrs.setErrorPage(defaultErrorPage);
		defaultAttrs.setFilterList(defaultFilterList);
		defaultAttrs.setGroupAttributes(defaultGroupAttrs);
		defaultAttrs.setAutoReload(defaultAutoReload);
		defaultAttrs.setReloadInterval(defaultReloadInterval);
		defaultAttrs.setEnabled(defaultEnabled);
		defaultAttrs.setURIPath(defaultGroupURI);

		Type typeObj = RepositoryObjectImpl.getTypeObj(myInterfaceName, myClassName, defaultAttrs, false);
		Long typeId = (Long)typeObj.getPrimaryKey();

		synchronized (classLock) 
		{
			myTypeObj = typeObj;
			myTypeId = typeId;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "initializeTypeId");
	}

	/*
	 * Methods mandated by the entity bean interface.
	 */

	/*
	 * ejbLoad -- Read from DB using primary key.
	 */
	public void ejbLoad() 
	throws RemoteException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "ejbLoad");

		Long primaryKey = (Long) ec.getPrimaryKey();
		DBQueryResult qs = null;

		try
		{
			qs = DBMgr.executePreparedReadByPrimaryKey(loadStmtKey,
																								 tableName,
																								 instanceIdColumnName,
																								 Utils.getIdString(primaryKey));

			ResultSet rs = qs.getResultSet();
			if (!rs.next())
			{
				RemoteException r = new RemoteException((nls.getFormattedMessage("segb.db.select1.exception",
																																				 new Object[] { primaryKey},
																																				 "Database select failed to find ServletGroup instance (id={0}).")),
																								new ObjectNotFoundException());
				if (tc.isEntryEnabled()) Tr.exit(tc, "ejbLoad", r);
				throw r;
			}

			if (setId(rs, myTypeId))
			{
				super.ejbLoad(); 

				description = rs.getString(descriptionColumnIndex);
				if (description == null)
				{
					description = new String();
				}
				docRoot = rs.getString(docRootColumnIndex);
				if (docRoot == null)
				{
					docRoot = new String();
				}
				classpath = (Vector)getBinaryAttr(classpathBinAttrName);
				sharedContext = Utils.intToBoolean(rs.getInt(sharedContextColumnIndex));
				sharedContextJNDIName = rs.getString(sharedContextJNDINameColumnIndex);
				if (sharedContextJNDIName == null)
				{
					sharedContextJNDIName = new String();
				}
				errorPage = rs.getString(errorPageColumnIndex);
				if (errorPage == null)
				{
					errorPage = new String();
				}
				filterList = (Properties)getBinaryAttr(filterListBinAttrName);
				groupAttributes = (Properties)getBinaryAttr(groupAttributesBinAttrName);
				autoReload = Utils.intToBoolean(rs.getInt(autoReloadColumnIndex));
				reloadInterval = rs.getInt(reloadIntervalColumnIndex);
				enabled = Utils.intToBoolean(rs.getInt(enabledColumnIndex));
				groupURI = rs.getString(groupURIColumnIndex);
			}

			// Reset local variables for related objects.
			containingObject = null;
			activeObjectName = null;
			virtualHost = null;

			// Attribute values are current.
			dirty = false;
			hasContext = true; 
		}
		catch (SQLException s)
		{
			RemoteException r = new RemoteException((nls.getString("segb.db.select2.exception",
																														 "Database select failed.")),
																							s);
			if (tc.isEntryEnabled()) Tr.exit(tc, "ejbLoad", r);
			throw r;
		}
		finally
		{
			DBMgr.doneWithQuery(qs);
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "ejbLoad");
	}

	/*
	 * ejbStore -- Store instance record in DB if any of the instance 
	 *             attributes had been modified.
	 */
	public void ejbStore() 
	throws RemoteException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "ejbStore");

		if (narrowRef != null)
		{
			if (tc.isEntryEnabled()) Tr.exit(tc, "ejbStore");
			return; 
		}

		super.ejbStore();		// live object attributes

		if (dirty)
		{
			try
			{
				Vector params = new Vector(numColumns);

				params.addElement(description);
				params.addElement(docRoot);
				params.addElement(new Integer(Utils.booleanToInt(sharedContext)));
				params.addElement(sharedContextJNDIName);
				params.addElement(errorPage);
				params.addElement(new Integer(Utils.booleanToInt(autoReload)));
				params.addElement(new Integer(reloadInterval));
				params.addElement(new Integer(Utils.booleanToInt(enabled)));
				params.addElement(groupURI);
				params.addElement(Utils.getIdString(id));

				DBMgr.executePreparedUpdate(updateStmtKey, updateStmtSql, params);
				dirty = false;
			}
			catch (SQLException s)
			{
				RemoteException r = new RemoteException((nls.getFormattedMessage("segb.db.update.exception",
																																				 new Object[] { id},
																																				 "Database update failed to store ServletGroup attributes (id={0}).")),
																								s); 
				if (tc.isEntryEnabled()) Tr.exit(tc, "ejbStore", r);
				throw r;
			}
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "ejbStore");
	}  

	/*
	 * ejbRemove -- Remove the instance record from DB.
	 */
	public void ejbRemove() 
	throws RemoteException, RemoveException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "ejbRemove");

		Long primaryKey = (Long) ec.getPrimaryKey();

		if (id == null)
		{
			ejbLoad();
		}

		if (narrowRef != null)
		{
			narrowRef.remove();
			if (tc.isEntryEnabled()) Tr.exit(tc, "ejbRemove");
			return;
		}

		// Need to determine if this instance is a Clone.
		boolean isClone = true;
		try
		{
			Model model = isAClone();
			if (model == null)
			{
				isClone = false;
			}
			else
			{
				isClone = true;
            ServletBeanHelper.reSet();//88536
			}
		}
		catch (OpException e)
		{
			// When in doubt, don't delete the URI!
			isClone = true;
		}

		// Send a refresh action of "REMOVE" to the Servlet Engine for this 
		// ServletGroup that is being removed.                              
		try
		{
			setRefreshConfigToEngine(ActiveServletGroup.REMOVE);
		}
		catch (OpException e)
		{
			ec.setRollbackOnly();
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.act.schedule1.exception",
																																			 new Object[] { id},
																																			 "Exception occurred scheduling After-Commit-Task to remove ServletGroup (id={0}).")),
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "ejbRemove", r);
			throw r;
		}

		super.ejbRemove();

		try
		{
			Vector params = new Vector(1);
			params.addElement(Utils.getIdString(primaryKey));

			DBMgr.executePreparedUpdate(deleteStmtKey, deleteStmtSql, params);
		}
		catch (SQLException s)
		{
			ec.setRollbackOnly();
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.db.remove.exception",
																																			 new Object[] { primaryKey},
																																			 "Database delete failed to remove ServletGroup instance (id={0}).")),
																							s);
			if (tc.isEntryEnabled()) Tr.exit(tc, "ejbRemove", r);
			throw r;
		}

		// If this ServletGroup is not a Clone, then remove the URI instance associated with it.
		if (!isClone)
		{
			URI uri = null;
			URIHome uriHome = (URIHome)javax.rmi.PortableRemoteObject.narrow(getHome("URIHome"), URIHome.class);

			try
			{
				uri = uriHome.findByName(groupURI, false);
				uri.setPrivateRemove(new Boolean(true));
				uri.remove();
			}
			catch (Exception e)
			{
			}
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "ejbRemove");
	}

	/*
	 * ejbCreate -- Insert record in DB after allocating id. 
	 */
	public Long ejbCreate(ServletGroupAttributes groupAttrs, ServletEngine engine)
	throws RemoteException, CreateException, AttributeNotSetException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "ejbCreate");

		// Create of new ServletGroup has no Context yet.
		hasContext = false;

		try
		{
			name = groupAttrs.getName();
			groupAttrs.getDocRoot();
			groupAttrs.getClasspath();
			groupAttrs.getURIPath();
		}
		catch (AttributeNotSetException e)
		{
			RequiredAttributeMissingException r = new RequiredAttributeMissingException(nls.getString("segb.attribute.missing1.exception",
																																																"Required ServletGroup name attribute was not specified."));
			if (tc.isEntryEnabled()) Tr.exit(tc,"ejbCreate -- required attribute missing", r);
			throw r;
		}

		try
		{
			// Check whether we are being invoked from a sub-class. 
			// If not, we are actually creating an instance of "this type".
			if (typeId == null)
			{
				typeId = myTypeId;
			}

			id = create(typeId); 

			super.ejbCreate();  

			// Need access to input ServletEngine.
			initialServletEngine = engine;

			// Allocate id and initialize instance variables. 
			initializeInstanceVariables();
			updateInstanceVariables(groupAttrs, false, false);

			// Add ServletEngine-ServletGroup relation. 
			Relation rel = lookupEngineGroupRel();

			try
			{
				rel.addUsingKeys((Long)engine.getPrimaryKey(), id, name);
			}
			catch (RelationOpException e)
			{
				InvalidServletGroupNameException i = new InvalidServletGroupNameException(nls.getFormattedMessage("segb.already.exists.exception",
																																																					new Object[] { name},
																																																					"ServletGroup already exists (name={0})."));
				if (tc.isEntryEnabled()) Tr.exit(tc, "ejbCreate", i);
				throw i;
			}

			// Insert record in database. 
			Vector params = new Vector(numColumns);

			params.addElement(Utils.getIdString(id));
			params.addElement(Utils.getIdString(typeId));
			params.addElement(description);
			params.addElement(docRoot);
			params.addElement(new Integer(Utils.booleanToInt(sharedContext)));
			params.addElement(sharedContextJNDIName);
			params.addElement(errorPage);
			params.addElement(new Integer(Utils.booleanToInt(autoReload)));
			params.addElement(new Integer(reloadInterval));
			params.addElement(new Integer(Utils.booleanToInt(enabled)));
			params.addElement(groupURI);

			DBMgr.executePreparedUpdate(insertStmtKey, insertStmtSql, params);
		}
		catch (SQLException s)
		{
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.db.insert.exception",
																																			 new Object[] { id},
																																			 "Database insert failed to store new ServletGroup instance (id={0}).")),
																							s);
			if (tc.isEntryEnabled()) Tr.exit(tc, "ejbCreate", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "ejbCreate", id);

		return id;
	}

	/*
	 * ejbCreate -- Insert record in DB after allocating id. 
	 */
	public Long ejbCreate(Model model, ServletGroupAttributes groupAttrs, ServletEngine engine)
	throws RemoteException, CreateException, AttributeNotSetException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "ejbCreate");

		// Create of new ServletGroup has no Context yet.
		hasContext = false;

		try
		{
			name = groupAttrs.getName();
			groupAttrs.getDocRoot();
			groupAttrs.getClasspath();
			groupAttrs.getURIPath();
		}
		catch (AttributeNotSetException e)
		{
			RequiredAttributeMissingException r = new RequiredAttributeMissingException(nls.getString("segb.attribute.missing1.exception",
																																																"Required ServletGroup name attribute was not specified."));
			if (tc.isEntryEnabled()) Tr.exit(tc,"ejbCreate -- required attribute missing", r);
			throw r;
		}

		try
		{
			// Check whether we are being invoked from a sub-class. 
			// If not, we are actually creating an instance of "this type".
			if (typeId == null)
			{
				typeId = myTypeId;
			}

			id = create(typeId); 

			super.ejbCreate();  

			// Need access to input ServletEngine.
			initialServletEngine = engine;

			// Allocate id and initialize instance variables. 
			initializeInstanceVariables();
			updateInstanceVariables(groupAttrs, true, false);

			// Add ServletEngine-ServletGroup relation. 
			Relation rel = lookupEngineGroupRel();

			try
			{
				rel.addUsingKeys((Long)engine.getPrimaryKey(), id, name);
			}
			catch (RelationOpException e)
			{
				InvalidServletGroupNameException i = new InvalidServletGroupNameException(nls.getFormattedMessage("segb.already.exists.exception",
																																																					new Object[] { name},
																																																					"ServletGroup already exists (name={0})."));
				if (tc.isEntryEnabled()) Tr.exit(tc, "ejbCreate", i);
				throw i;
			}

			// Insert record in database. 
			Vector params = new Vector(numColumns);

			params.addElement(Utils.getIdString(id));
			params.addElement(Utils.getIdString(typeId));
			params.addElement(description);
			params.addElement(docRoot);
			params.addElement(new Integer(Utils.booleanToInt(sharedContext)));
			params.addElement(sharedContextJNDIName);
			params.addElement(errorPage);
			params.addElement(new Integer(Utils.booleanToInt(autoReload)));
			params.addElement(new Integer(reloadInterval));
			params.addElement(new Integer(Utils.booleanToInt(enabled)));
			params.addElement(groupURI);

			DBMgr.executePreparedUpdate(insertStmtKey, insertStmtSql, params);
         ServletBeanHelper.reSet();//88536
		}
		catch (SQLException s)
		{
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.db.insert.exception",
																																			 new Object[] { id},
																																			 "Database insert failed to store new ServletGroup instance (id={0}).")),
																							s);
			if (tc.isEntryEnabled()) Tr.exit(tc, "ejbCreate", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "ejbCreate", id);

		return id;
	}

	/*
	 * ejbPostCreate -- Post ejbCreate() opportunity.
	 */
	public void ejbPostCreate(ServletGroupAttributes groupAttrs,  ServletEngine engine) 
	{
	} 

	public void ejbPostCreate(Model model, ServletGroupAttributes groupAttrs,  ServletEngine engine) 
	{
	} 

	/*
	 * Finder methods.
	 */

	/*
	 * ejbFindAll -- Return enumeration of all DB table entries.
	 */
	public Enumeration ejbFindAll(boolean useSubtypes) 
	throws RemoteException, FinderException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "ejbFindAll");

		int stmtKey;

		if (useSubtypes)
		{
			stmtKey = findAllStmtKey;
		}
		else
		{
			stmtKey = restrictedFindAllStmtKey;
		}

		Enumeration keyList;

		try
		{
			keyList = DBMgr.executeFindAll(stmtKey, 
																		 tableName,
																		 useSubtypes,
																		 typeIdColumnName,
																		 myTypeId,
																		 instanceIdColumnName);
		}
		catch (SQLException s)
		{
			RemoteException r = new RemoteException((nls.getString("segb.db.select3.exception",
																														 "Database select failed to find all instances of ServletGroup.")),
																							s);
			if (tc.isEntryEnabled()) Tr.exit(tc, "ejbFindAll", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "ejbFindAll", keyList);
		return keyList;
	}

	/*
	 * initializeInstanceVariables -- Initialize instance variables.
	 */
	private void initializeInstanceVariables() 
	throws RemoteException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "initializeInstanceVariables");

		try
		{
			ServletGroupAttributes defaultAttrs = (ServletGroupAttributes) RepositoryObjectImpl.getDefaults(myTypeObj);

			description = defaultAttrs.getDescription();
			docRoot = defaultAttrs.getDocRoot(); 
			classpath = defaultAttrs.getClasspath();
			setBinaryAttr(classpathBinAttrName, classpath);
			sharedContext = defaultAttrs.getSharedContext();
			sharedContextJNDIName = defaultAttrs.getSharedContextJNDIName();
			errorPage = defaultAttrs.getErrorPage();
			filterList = defaultAttrs.getFilterList();
			setBinaryAttr(filterListBinAttrName, filterList);
			groupAttributes = defaultAttrs.getGroupAttributes();
			setBinaryAttr(groupAttributesBinAttrName, groupAttributes);
			autoReload = defaultAttrs.getAutoReload();
			reloadInterval = defaultAttrs.getReloadInterval();
			enabled = defaultAttrs.getEnabled();
			groupURI = defaultAttrs.getURIPath();
		}
		catch (Exception e)
		{
			RemoteException r = new RemoteException((nls.getString("segb.attribute.init.exception",
																														 "Exception occurred initializing default ServletGroup bean attributes.")),
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "initializeInstanceVariables", r);
			throw r;
		}

		// Reset local variables for related objects.
		containingObject = null;
		activeObjectName = null;
		virtualHost = null;

		if (tc.isEntryEnabled()) Tr.exit(tc, "initializeInstanceVariables");
	}

	/*
	 * updateInstanceVariables -- Update instance variables.
	 */
	private void updateInstanceVariables(ServletGroupAttributes groupAttrs, boolean isCloneCreate, boolean isCloneInstance) 
	throws RemoteException, InvalidURINameException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "updateInstanceVariables");

		dirty = true;

		try
		{
			if (groupAttrs.isSet(ServletGroupAttributes.description))
				description = groupAttrs.getDescription();
			if (groupAttrs.isSet(ServletGroupAttributes.docRoot))
			{
				docRoot = groupAttrs.getDocRoot();
				docRoot = URIBean.platformSlashSwitch(docRoot);
			}
			if (groupAttrs.isSet(ServletGroupAttributes.classpath))
			{
				classpath = groupAttrs.getClasspath();
				int csize = classpath.size();

				Vector cpath = new Vector(csize);

				for (int i = 0 ; i < csize; i++)
				{
					String currentPath = (String)(classpath.elementAt(i));
					String checkPath = URIBean.platformSlashSwitch(currentPath);
					cpath.addElement(checkPath);
				}

				classpath = cpath;
				setBinaryAttr(classpathBinAttrName, classpath);
			}
			if (groupAttrs.isSet(ServletGroupAttributes.sharedContext))
				sharedContext = groupAttrs.getSharedContext();
			if (groupAttrs.isSet(ServletGroupAttributes.sharedContextJNDIName))
				sharedContextJNDIName = groupAttrs.getSharedContextJNDIName();
			if (groupAttrs.isSet(ServletGroupAttributes.errorPage))
				errorPage = groupAttrs.getErrorPage();
			if (groupAttrs.isSet(ServletGroupAttributes.filterList))
			{
				filterList = groupAttrs.getFilterList();
				setBinaryAttr(filterListBinAttrName, filterList);
			}
			if (groupAttrs.isSet(ServletGroupAttributes.groupAttributes))
			{
				groupAttributes = groupAttrs.getGroupAttributes();
				setBinaryAttr(groupAttributesBinAttrName, groupAttributes);
			}
			if (groupAttrs.isSet(ServletGroupAttributes.autoReload))
				autoReload = groupAttrs.getAutoReload();
			if (groupAttrs.isSet(ServletGroupAttributes.reloadInterval))
				reloadInterval = groupAttrs.getReloadInterval();
			if (groupAttrs.isSet(ServletGroupAttributes.enabled))
				enabled = groupAttrs.getEnabled();
			if (groupAttrs.isSet(ServletGroupAttributes.uriPath))
			{
				if (isCloneCreate)
				{
					groupURI = groupAttrs.getURIPath();
				}
				else
				{
					setURI(groupAttrs.getURIPath(), isCloneInstance);
				}
			}
		}
		catch (AttributeNotSetException e)
		{
			RemoteException r = new RemoteException((nls.getString("segb.attribute.missing2.exception",
																														 "Required attribute for ServletGroup was not input.")),
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "updateInstanceVariables", r);
			throw r;
		}
		catch (AttributeDoesNotExistException e)
		{
			RemoteException r = new RemoteException((nls.getString("segb.attribute.not.exist.exception",
																														 "Input attribute does not exist for a ServletGroup.")),
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "updateInstanceVariables", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "updateInstanceVariables");
	}

	/*
	 * readInstanceVariables -- Return current instance variables.
	 */
	private ServletGroupAttributes readInstanceVariables(Attributes attributeNames) 
	throws RemoteException, OpException 
	{ 
		if (tc.isEntryEnabled()) Tr.entry(tc, "readInstanceVariables");

		ServletGroupAttributes groupAttrs = (ServletGroupAttributes) attributeNames;

		groupAttrs = (ServletGroupAttributes) super.getAttributes(groupAttrs);
		groupAttrs.setName(getRelativeName());
		groupAttrs.setDescription(description);
		groupAttrs.setDocRoot(docRoot);
		groupAttrs.setClasspath(classpath);
		groupAttrs.setSharedContext(sharedContext);
		groupAttrs.setSharedContextJNDIName(sharedContextJNDIName);
		groupAttrs.setErrorPage(errorPage);
		groupAttrs.setFilterList(filterList);
		groupAttrs.setGroupAttributes(groupAttributes);
		groupAttrs.setAutoReload(autoReload);
		groupAttrs.setReloadInterval(reloadInterval);
		groupAttrs.setEnabled(enabled);
		groupAttrs.setURIPath(groupURI);

		if (groupAttrs.activeAttributeRequested())
		{
			getActiveAttributes(attributeNames);
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "readInstanceVariables");
		return groupAttrs;
	}

	/*
	 * Relationship Methods
	 */

	/*
	 * lookupEngineGroupRel -- create the Servlet Engine-Group relationship if necessary. 
	 */
	private static Relation lookupEngineGroupRel() 
	throws RemoteException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "lookupServletGroupRel");

		synchronized (classLock) 
		{
			if (engineGroupRel != null)
			{
				if (tc.isEntryEnabled()) Tr.exit(tc, "lookupServletGroupRel");
				return engineGroupRel;
			}
		}

		Relation rel = ServletEngineBean.lookupEngineGroupRel();

		synchronized (classLock) 
		{
			engineGroupRel = rel;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "lookupServletGroupRel");
		return rel;
	}

	/*
	 * lookupGroupServletRel -- create the Servlet Group-Servlet 
	 *                          relationship if necessary. 
	 */
	protected static Relation lookupGroupServletRel()
	throws RemoteException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "lookupGroupServletRel");

		synchronized (classLock) 
		{
			if (groupServletRel != null)
			{
				if (tc.isEntryEnabled()) Tr.exit(tc, "lookupGroupServletRel");
				return groupServletRel;
			}
		}

		Relation rel = RepositoryObjectImpl.createRelIfNeeded(myTypeObj,
																													"Servlet", 
																													groupServletRelName,  
																													true, true,  
																													RelationCardinality.ONE_TO_MANY); 

		synchronized (classLock) 
		{
			groupServletRel = rel;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "lookupGroupServletRel");
		return rel;
	}

	/*
	 * lookupApplicationBeanRel -- Lookups the ejbContainer bean relation by
	 * querying the relation home. 
	 */
	private static Relation lookupApplicationServletGroupRel()  
	throws RemoteException {
		synchronized(classLock) {
			if (applicationServletGroupRel != null)
				return applicationServletGroupRel;
		}
		if (tc.isEntryEnabled())Tr.entry(tc, "lookupApplicationServletGroupRel");

		Type srcType = RepositoryObjectImpl.getTypeObj(RepositoryObjectImpl.qualifyClassName("Application"),
																									 RepositoryObjectImpl.qualifyClassName("ApplicationBean"),
																									 null, false);
		Relation rel = RepositoryObjectImpl.createRelIfNeeded(srcType, 
																													"ServletGroup", 
																													applicationServletGroupRelName,  
																													false, false,  RelationCardinality.MANY_TO_MANY); 

		synchronized(classLock) {
			applicationServletGroupRel = rel;
		}
		Tr.exit(tc, "lookupApplicationServletGroupRel");
		return rel;
	}

	/*
	 * getRelativeName -- Get the name from the relation.
	 */
	protected String getRelativeName() 
	throws RemoteException 
	{
		return super.getName(lookupEngineGroupRel());
	}

	/*
	 * getFullName -- Get the fully-qualified name.
	 */
	public RepositoryObjectName getFullName()
	throws RemoteException, OpException
	{
		return constructFullName("ServletGroupHome", getRelativeName());
	}

	/*
	 * getContainingObject -- Get ServletEngine that contains this ServletGroup. 
	 */ 
	public EJBObject getContainingObject() 
	throws RemoteException, ObjectNotFoundException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getContainingObject");

		if (containingObject != null)
		{
			if (tc.isEntryEnabled()) Tr.exit(tc, "getContainingObject");
			return containingObject;
		}

		Relation rel = lookupEngineGroupRel();
		ServletEngine servletEngine;

		Long primaryKey = (Long) ec.getPrimaryKey();

		try
		{
			servletEngine = (ServletEngine)javax.rmi.PortableRemoteObject.narrow(rel.traverse(ec.getEJBObject(), true), ServletEngine.class);
		}
		catch (RelationOpException e)
		{
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.relation.traverse1.exception",
																																			 new Object[] { primaryKey},
																																			 "Exception occurred while traversing Relation instances looking for ServletEngine for the ServletGroup (id={0}).")), 
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "getContainingObject", r);
			throw r;
		}

		containingObject = servletEngine;

		if (tc.isEntryEnabled()) Tr.exit(tc, "getContainingObject");
		return servletEngine;
	}

	/*
	 * ActiveObject Methods
	 */

	/*
	 * getActiveObjectName -- Return the Active Object for this Servlet Engine
	 *                        instance.
	 */
	public ContainmentPathElem getActiveObjectName() 
	throws RemoteException, OpException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getActiveObjectName");

		if (activeObjectName != null)
		{
			if (tc.isEntryEnabled()) Tr.exit(tc, "getActiveObjectName");
			return activeObjectName;
		}

		activeObjectName = new ContainmentPathElem(id,
																							 activeObjectClassName,
																							 getRelativeName(),
																							 getEpoch());

		if (tc.isEntryEnabled()) Tr.exit(tc, "getActiveObjectName");
		return activeObjectName;
	}

	/*
	 * getConfig -- Return the the ActiveServletGroup config. 
	 */
	public ActiveObjectConfig getConfig() 
	throws RemoteException, OpException 
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getConfig");

		ActiveServletGroupConfig config = new ActiveServletGroupConfig();

		super.initializeConfig(config);

		config.setDescription(description);
		config.setDocRoot(docRoot);
		config.setClasspath(classpath);
		config.setSharedContext(sharedContext);
		config.setSharedContextJNDIName(sharedContextJNDIName);
		config.setErrorPage(errorPage);
		config.setFilterList(filterList);
		config.setGroupAttributes(groupAttributes);
		config.setAutoReload(autoReload);
		config.setReloadInterval(reloadInterval);
		config.setEnabled(enabled);
		config.setURIPath(groupURI);

		if (tc.isEntryEnabled()) Tr.exit(tc, "getConfig");
		return config;
	}

	/* 
	 * Superclass Overrides
	 */
	/*
	public Act start()
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "start");

		super.start();

		if (tc.isEntryEnabled()) Tr.exit(tc, "start");
	}
	*/

	/*
	 * ServletGroup Interface Methods
	 */

	/*
	 * setEnabled - Schedules an After-Commit-Task for invoking
	 *              the setEnabled() method of ActiveServletGroup.
	 */
	public Act setEnabled(Boolean eFlag)
	throws RemoteException, OpException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "setEnabled");

		String activeMethod = "setEnabledAction";
		boolean enabledFlag = eFlag.booleanValue();

		try
		{
			if ((enabled != enabledFlag) && (getDesiredState() == ActiveObjectStatus.RUNNING))
			{
				AdminAgent nodeAgent = getNode().getAdminAgent();
				ContainmentPath target = getActiveObjectContainmentPath();
				ActServerImpl actServer = ActServerImpl.getInstance();   

				VirtualHost vhost = getVirtualHost();
				String vhostName = null;

				if (vhost != null)
				{
					VirtualHostAttributes vhostAttrs = new VirtualHostAttributes();
					vhostAttrs = (VirtualHostAttributes) (vhost.getAttributes(vhostAttrs));
					vhostName = vhostAttrs.getName();
				}

				int epoch = super.getEpoch();
				int version = super.getVersion();

				ParamList paramList = new ParamList(4);
				paramList.addElement(enabledFlag);
				paramList.addElement(vhostName);
				paramList.addElement(epoch);
				paramList.addElement(version);

				// Indicates to ejbStore() to persist the enabled state.
				enabled = enabledFlag;
				dirty = true;

				Act act = actServer.scheduleWork(new ActionTask(nodeAgent, 
																												target,
																												activeMethod,
																												paramList,
																												(EJBObject)ec.getEJBObject()));

				if (tc.isEntryEnabled()) Tr.exit(tc, "setEnabled");
				return act;
			}
		}
		catch (OpException e)
		{
			ServletGroupSetEnabledException s = new ServletGroupSetEnabledException(nls.getFormattedMessage("segb.act.schedule2.exception",
																																																			new Object[] { activeMethod},
																																																			"Exception occurred scheduling After-Commit-Task (method={0})."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setEnabled", s);
			throw s;
		}
		catch (RemoteException e)
		{
			ServletGroupSetEnabledException s = new ServletGroupSetEnabledException(nls.getFormattedMessage("segb.act.schedule2.exception",
																																																			new Object[] { activeMethod},
																																																			"Exception occurred scheduling After-Commit-Task (method={0})."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setEnabled", s);
			throw s;
		}
		catch (AttributeNotSetException e)
		{
			ServletGroupSetEnabledException s = new ServletGroupSetEnabledException(nls.getString("segb.attribute.not.set1.exception",
																																														"VirtualHost name attribute was not set."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setEnabled", s);
			throw s;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "setEnabled - ServletGroup is not RUNNING, returning null.");
		return null;
	}

	/*
	 * setRefreshConfig - Invokes the ActiveServletGroup setRefreshConfig() method
	 *                    to initiate an action on this ServletGroup.
	 */
	public Act setRefreshConfig(Integer rAction)
	throws RemoteException, OpException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "setRefreshConfig");

		String activeMethod = "setRefreshConfigAction";
		int refreshAction = rAction.intValue();

		Act act = null;

		try
		{
			// Determine if the ServletEngine is running. If it is, then start this ServletGroup
			// if it is not currently running. Else, do nothing since the Server has to be started.
			ServletEngine engine = (ServletEngine) getContainingObject();

			if (engine.getDesiredState() == ActiveObjectStatus.RUNNING)
			{
				if (getDesiredState() != ActiveObjectStatus.RUNNING)
				{
					act = start();
				}
			}

			if (getDesiredState() == ActiveObjectStatus.RUNNING)
			{
				AdminAgent nodeAgent = getNode().getAdminAgent();
				ContainmentPath target = getActiveObjectContainmentPath();
				ActServerImpl actServer = ActServerImpl.getInstance();   

				ActiveObjectConfig config = getConfig();

				// Get all contained Servlets for this ServletGroup.
				TypeHome typeHome = (TypeHome) getHome("TypeHome");
				Type servletType = typeHome.findByImplClass("com.ibm.ejs.sm.beans.ServletBean", true);

				Enumeration servlets = listContainedObjects(servletType);
				while (servlets.hasMoreElements())
				{
					Servlet servlet = (Servlet)javax.rmi.PortableRemoteObject.narrow(servlets.nextElement(), 
																																					 Servlet.class);

					// If the Servlet is not RUNNING, start it now!
					if (servlet.getDesiredState() != ActiveObjectStatus.RUNNING)
					{
						servlet.start();
					}

					// Refresh the Active attributes.
					servlet.setRefreshConfig(new Integer(ActiveServlet.REFRESH));

					// Get the refreshed ActiveServletConfig.
					ActiveObjectConfig servletConfig = servlet.getActiveServletConfig(); 

					// Add the ActiveServletConfig child to this ActiveServletGroupConfig.
					config.addContainedConfig(servletConfig); 
				}

				VirtualHost vhost = getVirtualHost();
				String vhostName = null;

				if (vhost != null)
				{
					VirtualHostAttributes vhostAttrs = new VirtualHostAttributes();
					vhostAttrs = (VirtualHostAttributes) (vhost.getAttributes(vhostAttrs));
					vhostName = vhostAttrs.getName();
				}

				int epoch = super.getEpoch();
				int version = super.getVersion();

				ParamList paramList = new ParamList(5);
				paramList.addElement(refreshAction);
				paramList.addElement(config);
				paramList.addElement(vhostName);
				paramList.addElement(epoch);
				paramList.addElement(version);

				act = actServer.scheduleWork(new ActionTask(nodeAgent, 
																										target,
																										activeMethod,
																										paramList,
																										(EJBObject)ec.getEJBObject()));
			}
		}
		catch (OpException e)
		{
			ServletGroupSetRefreshConfigException s = new ServletGroupSetRefreshConfigException(nls.getFormattedMessage("segb.act.scehdule2.exception",
																																																									new Object[] { activeMethod},
																																																									"Exception occurred scheduling After-Commit-Task (method={0})."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfig", s);
			throw s;
		}
		catch (RemoteException e)
		{
			ServletGroupSetRefreshConfigException s = new ServletGroupSetRefreshConfigException(nls.getFormattedMessage("segb.act.schedule2.exception",
																																																									new Object[] { activeMethod},
																																																									"Exception occurred scheduling After-Commit-Task (method={0})."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfig", s);
			throw s;
		}
		catch (FinderException e)
		{
			ServletGroupSetRefreshConfigException s = new ServletGroupSetRefreshConfigException(nls.getString("segb.servlet.not.found.exception",
																																																				"Exception occurred trying to find ServletBean Type instance."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfig", s);
			throw s;
		}
		catch (AttributeNotSetException e)
		{
			ServletGroupSetRefreshConfigException s = new ServletGroupSetRefreshConfigException(nls.getString("segb.attribute.not.set1.exception",
																																																				"VirtualHost name attribute was not set."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfig", s);
			throw s;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfig");
		return act;
	}

	/*
	 * setRefreshConfigToEngine - Invokes the ActiveServletEngine setRefreshConfig() method
	 *                            to initiate an action on the input ServletGroup.
	 */
	public Act setRefreshConfigToEngine(int refreshAction)
	throws RemoteException, OpException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "setRefreshConfigToEngine");

		Long primaryKey = (Long) ec.getPrimaryKey();

		try
		{
			VirtualHost vhost = getVirtualHost();
			String vhostName = null;

			if (vhost != null)
			{
				VirtualHostAttributes vhostAttrs = new VirtualHostAttributes();
				vhostAttrs = (VirtualHostAttributes) (vhost.getAttributes(vhostAttrs));
				vhostName = vhostAttrs.getName();
			}

			String groupName = getRelativeName();
			ServletEngine servletEngine = (ServletEngine) getContainingObject();

			Act act = servletEngine.setRefreshConfig(refreshAction, vhostName, groupName);

			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfigToEngine");
			return act;
		}
		catch (OpException e)
		{
			ServletGroupSetRefreshConfigException s = new ServletGroupSetRefreshConfigException(nls.getString("segb.act.schedule3.exception",
																																																				"Exception occurred calling ServletEngine to schedule After-Commit-Task."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfigToEngine", s);
			throw s;
		}
		catch (RemoteException e)
		{
			ServletGroupSetRefreshConfigException s = new ServletGroupSetRefreshConfigException(nls.getString("segb.act.schedule4.exception",
																																																				"Exception occurred calling ServletEngine scheduling After-Commit-Task."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfigToEngine", s);
			throw s;
		}
		catch (ObjectNotFoundException e)
		{
			ServletGroupSetRefreshConfigException s = new ServletGroupSetRefreshConfigException(nls.getFormattedMessage("segb.engine.not.found.exception",
																																																									new Object[] { primaryKey},
																																																									"No ServletEngine was found for ServletGroup (id={0})."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfigToEngine", s);
			throw s;
		}
		catch (AttributeNotSetException e)
		{
			ServletGroupSetRefreshConfigException s = new ServletGroupSetRefreshConfigException(nls.getString("segb.attribute.not.set1.exception",
																																																				"VirtualHost name attribute was not set."));
			e.printStackTrace();
			if (tc.isEntryEnabled()) Tr.exit(tc, "setRefreshConfigToEngine", s);
			throw s;
		}
	}

	/*
	 * getVirtualHost - Returns the VirtualHost associated with this ServletGroup.
	 */
	public VirtualHost getVirtualHost()
	throws RemoteException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getVirtualHost");

		if (virtualHost != null)
		{
			if (tc.isEntryEnabled()) Tr.exit(tc, "getVirtualHost");
			return virtualHost;
		}

		Long primaryKey = (Long) ec.getPrimaryKey();

		VirtualHost virtualHost = null;
		String groupVirtualHostName = null;

		int firstSlash = groupURI.indexOf("/");
		if (firstSlash >= 0)
		{
			groupVirtualHostName = groupURI.substring(0, firstSlash);
		}
		else
		{
			groupVirtualHostName = groupURI;
		}

		try
		{
			VirtualHostHome virtualHostHome = (VirtualHostHome)javax.rmi.PortableRemoteObject.narrow(getHome("VirtualHostHome"),
																																															 VirtualHostHome.class);
			virtualHost = virtualHostHome.findByName(groupVirtualHostName, false);
		}
		catch (FinderException e)
		{
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.virtualhost.not.found.exception",
																																			 new Object[] { primaryKey},
																																			 "Exception occurred attempting to get the VirtualHost for the ServletGroup (id={0}).")),
																							e);


			if (tc.isEntryEnabled()) Tr.exit(tc, "getVirtualHost", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "getVirtualHost");
		return virtualHost;
	}

	/*
	 * getURIPath - Returns the full path, including the VirtualHost
	 *              name, for the URI that this ServletGroup is associated
	 *              with. The format of the path is:
	 *                  <virtualhost><root_uri>
	 */
	public String getURIPath() 
	throws RemoteException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getURIPath");
		return groupURI;
	}

	/*
	 * getURI - Returns the URI associated with this ServletGroup, if it exists.
	 */
	public URI getURI()
	throws RemoteException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getURI");

		Long primaryKey = (Long) ec.getPrimaryKey();
		URI uri = null;

		try
		{
			URIHome uriHome = (URIHome)javax.rmi.PortableRemoteObject.narrow(getHome("URIHome"), URIHome.class);
			uri = uriHome.findByName(groupURI, false);
		}
		catch (FinderException e)
		{
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.uri.not.found1.exception",
																																			 new Object[] { primaryKey},
																																			 "Exception occurred attempting to get the URI for the ServletGroup (id={0}).")),
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "getVirtualHost", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "getURI");
		return uri;
	}

	/*
	 * setURI - Sets the ServletGroup URI attribute and creates the corresponding URI 
	 *          instance for the input URI path. The input URI path format is:
	 *              <virtualhost><root_uri>
	 */
	public void setURI(String newURIPath, boolean isCloneInstance)
	throws RemoteException, InvalidURINameException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "setURI");

		Long primaryKey = null;
		String uriPath = null;
		String oldURIPath = null;
		URI currentURI = null;
		URI newURI = null;
		Long currentKey = null;
		Long newKey = null;
		int uriType = 0;

		ServletEngine servletEngine = null;
		Servlet servlet = null;
		Vector servletList = null;
		ServletGroup servletGroup = null;
		Vector groupList = null;
		int listSize = 0;
      //@88536 Added - start
      boolean skipForClone=false;

      if (isCloneInstance) {
      
      skipForClone=ServletBeanHelper.getDeleteFlag((Long)ec.getPrimaryKey(),this);
      
      
      //System.out.println("SGB:: skipForClone is "+skipForClone);
       //@88536A end
      }

		try
		{
			uriPath = newURIPath;

			// If no URI path were input, then throw exception.
			if (uriPath == null)
			{
				RemoteException r = new RemoteException(nls.getString("segb.seturi.null.uri.exception",
																															"No URI path was input for the ServletGroup."));
				if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
				throw r;
			}

			uriPath = URIBean.removeDoubleSlash(newURIPath);

			String inVirtualHostName = null;
			String inURIPath = null;        

			int firstSlash = uriPath.indexOf("/");
			if (firstSlash >= 0)
			{
				inVirtualHostName = uriPath.substring(0, firstSlash);
				inURIPath = uriPath.substring(firstSlash);
			}
			else
			{
				RemoteException r = new RemoteException(nls.getFormattedMessage("segb.seturi.bad.uri.exception",
																																				new Object[] { uriPath},
																																				"Incorrect input URI path format (path={0})."));
				if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
				throw r;
			}

			URIHome uriHome = (URIHome)javax.rmi.PortableRemoteObject.narrow(getHome("URIHome"), URIHome.class);

			// Determine if the given URI already exists.
			try
			{
				newURI = uriHome.findByName(uriPath, false);
				newKey = (Long) (newURI.getPrimaryKey());

				// If the input URI was found, and this is not a ServletGroup Clone,
				// then determine if it already belongs to a Servlet or ServletGroup. 
				// If it does, then throw the URI already exsist exception. Else, just
				// use the unused URI.
				if (!isCloneInstance)
				{
					try
					{
						servletList = newURI.getAllServlets();
					}
					catch (Exception e)
					{
					}

					try
					{
						groupList = newURI.getAllServletGroups();
					}
					catch (Exception e)
					{
					}

					Node thisNode = null;
					Long thisNodeKey = null;
					Node servletNode = null;
					Long servletNodeKey = null;
					Node servletGroupNode = null;
					Long servletGroupNodeKey = null;

					boolean uriExists = false;
					InvalidURINameException iune = null;
					String fullName = null;

					if (!servletList.isEmpty())
					{
						listSize = servletList.size();
						for (int s = 0; s < listSize; s++)
						{
							servlet = (Servlet) servletList.elementAt(s);
							servletNode = servlet.getNode();
							servletNodeKey = (Long)(servletNode.getPrimaryKey());

							if (!hasContext)
							{
								servletEngine = initialServletEngine;
								thisNode = servletEngine.getNode();
								thisNodeKey = (Long)(thisNode.getPrimaryKey());
							}
							else
							{
								thisNode = getNode();
								thisNodeKey = (Long)(thisNode.getPrimaryKey());
							}

							// If a Servlet was found for the URI and it is on the same Node that "this" ServletGroup 
							// is, then throw exception.
							if ((servletNodeKey.intValue()) == (thisNodeKey.intValue()))
							{
							  // Special Case:
								//  If the new URI for this servlet group happens to be '/', then we must
								//  allow this URI to have a double meaning. One as the Root URI AND
								//  the other as a Servlet URI.
								if (!(inURIPath.equals("/")))
								{
								  uriExists = true;
								  fullName = (servlet.getFullName()).toString();
								  iune = new InvalidURINameException(nls.getFormattedMessage("segb.seturi.already.exists1.exception",
									  																												 new Object[] { fullName},
										  																											 "URI specified already exists for resource (Servlet={0})."));
								}
							}
						}
					}
					else if (!groupList.isEmpty())
					{
						listSize = groupList.size();
						for (int s = 0; s < listSize; s++)
						{
							servletGroup = (ServletGroup) groupList.elementAt(s);
							servletGroupNode = servletGroup.getNode();
							servletGroupNodeKey = (Long)(servletGroupNode.getPrimaryKey());

							if (!hasContext)
							{
								servletEngine = initialServletEngine;
								thisNode = servletEngine.getNode();
								thisNodeKey = (Long)(thisNode.getPrimaryKey());

								// If the ServletGroup found for the URI is on the same Node that "this" ServletGroup 
								// is, then throw exception.
								if ((servletGroupNodeKey.intValue()) == (thisNodeKey.intValue()))
								{
									uriExists = true;
									fullName = (servletGroup.getFullName()).toString();
									iune = new InvalidURINameException(nls.getFormattedMessage("segb.seturi.already.exists2.exception",
																																						 new Object[] { fullName},
																																						 "URI specified already exists for resource (Web Application={0})."));
								}
							}
							else
							{
								thisNode = getNode();
								thisNodeKey = (Long)(thisNode.getPrimaryKey());

								primaryKey = (Long)(ec.getPrimaryKey());
								Long servletGroupKey = (Long)(servletGroup.getPrimaryKey());

								// If the ServletGroup found for the URI is on the same Node that "this" ServletGroup 
								// is AND it is not looking itself, then throw exception.
								if (((servletGroupNodeKey.intValue()) == (thisNodeKey.intValue())) &&
										((primaryKey.intValue()) != (servletGroupKey.intValue())))
								{
									uriExists = true;
									fullName = (servletGroup.getFullName()).toString();
									iune = new InvalidURINameException(nls.getFormattedMessage("segb.seturi.already.exists2.exception",
																																						 new Object[] { fullName},
																																						 "URI specified already exists for resource (Web Application={0})."));
								}
							}
						}
					}

					if (uriExists)
					{
						if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", iune);
						throw iune;
					}
				}
			}
			catch (ObjectNotFoundException e)
			{
				newURI = null;
			}

			URIAttributes uriAttributes = new URIAttributes();

			if (newURI == null)
			{
				// Create a new URI.
				VirtualHostHome virtualHostHome = (VirtualHostHome)javax.rmi.PortableRemoteObject.narrow(getHome("VirtualHostHome"), 
																																																 VirtualHostHome.class);

				VirtualHost virtualHost = null;

				try
				{
					virtualHost = virtualHostHome.findByName(inVirtualHostName, false);
				}
				catch (ObjectNotFoundException e)
				{
					ObjectNotFoundException o = new ObjectNotFoundException(nls.getFormattedMessage("segb.seturi.virtualhost.not.found.exception",
																																													new Object[] { inVirtualHostName},
																																													"VirtualHost not found (name={0})."));
					if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", o);
					throw o;
				}

				uriAttributes.setName(inURIPath);

				try
				{
					newURI = uriHome.create(uriAttributes, virtualHost);
				}
				catch (CreateException e)
				{
					RemoteException r =  new RemoteException((nls.getFormattedMessage("segb.seturi.uri.create1.exception",
																																						new Object[] { uriPath},
																																						"Exception occurred creating URI instance for input path (path={0}).")), 
																									 e);
					if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
					throw r;
				}
			}
			else
			{
				// Found an eligible URI, so refresh it's attributes!
				uriAttributes = (URIAttributes)(newURI.getAttributes(uriAttributes));
				uriAttributes.setPathName(inURIPath);
				uriAttributes.setRootURI(inURIPath);
				newURI.setAttributes(uriAttributes);
			}

			// Case 1: If this is an initial create, then initialize the local groupURI instance variable.
			if (!hasContext)
			{
				// Initialize the groupURI instance variable.
				groupURI = uriPath;
			}

			// Case 2: This is a Root URI change for this ServletGroup, then update all the 
			// update all the Servlet URIs associated with this ServletGroup and 
			// then remove the old Root URI.
			if (hasContext)
			{
				// This is not an initial create.

				// Update the groupURI instance variable.
				if (groupURI.equals(uriPath))
				{
					return;
				}

				oldURIPath = groupURI;
				groupURI = uriPath;

				// Update all the URIs for all the Servlets under this ServletGroup
				// with the new root URI.
         if (skipForClone) {   //RK
            try {
               updateServletURIs(oldURIPath, groupURI, isCloneInstance);
            } catch (RemoteException e) {
               RemoteException r = new RemoteException((nls.getString("segb.seturi.servlet.update.exception",
                                                                      "Update of the URI(s) for Servlet(s) for this ServletGroup failed.")),
                                                       e);
               if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
               throw r;
            }
         }
				// Persist the groupURI instance variable. 
				dirty = true;
				ejbStore();

				// Get current URI for this group.
				try
				{
					currentURI = uriHome.findByName(oldURIPath, false);
					currentKey = (Long) (currentURI.getPrimaryKey());
				}
				catch (ObjectNotFoundException e)
				{
					currentURI = null;
				}

				// Remove the URI only if this ServletGroup is NOT a Clone!
            //if ((currentURI != null) && (!isCloneInstance)) { //@88536 Deleted

            if ((currentURI != null) && (skipForClone)) { //@88536 Changed
					currentURI.setPrivateRemove(new Boolean(true));
					currentURI.remove();
				}
			}
		}
		catch (OpException e)
		{
			RemoteException r =  new RemoteException((nls.getString("segb.seturi.uri.create.delete.exception",
																															"Exception occurred creating/deleting a URI instance.")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
			throw r;
		}
		catch (FinderException e)
		{
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.seturi.uri.find.exception",
																																			 new Object[] { uriPath},
																																			 "Exception occurred attempting to find URI instance (path={0}).")),
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
			throw r;
		}
		catch (RemoveException e)
		{
			RemoteException r = new RemoteException((nls.getFormattedMessage("segb.seturi.uri.remove.exception",
																																			 new Object[] { uriPath},
																																			 "Exception occurred attempting remove URI instance (path={0}).")),
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
			throw r;
		}
		catch (RemoteException e)
		{
			RemoteException r =  new RemoteException((nls.getString("segb.seturi.uri.create2.exception",
																															"Exception occurred creating new URI instance.")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
			throw r;
		}
		catch (AttributeNotSetException e)
		{
			RemoteException r =  new RemoteException((nls.getString("segb.seturi.uri.create.delete.exception",
																															"Exception occurred creating/deleting a URI instance.")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "setURI", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "setURI");
	}

	/*
	 * renameURIVirtualHost -- Renames the fully-qualified URI path for the ServletGroup
	 *                         from the old VirtualHost name to the new input VirtualHost name.
	 *                         In addition, the URIs for all Servlets under this ServletGroup
	 *                         are renamed.
	 */
	public void renameURIVirtualHost(String oldVirtualHostName, String newVirtualHostName)
	throws RemoteException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "renameURIVirtualHost");

		virtualHost = null;

		try
		{
			String newURI = null;
			String currentVirtualHostName = null;
			String currentURIPath = null;

			int firstSlash = groupURI.indexOf("/");
			if (firstSlash >= 0)
			{
				currentVirtualHostName = groupURI.substring(0, firstSlash);
				currentURIPath = groupURI.substring(firstSlash);
			}
			else
			{
				RemoteException r = new RemoteException(nls.getFormattedMessage("segb.renameurivh.bad.uri.exception",
																																				new Object[] { groupURI},
																																				"Incorrect URI path format found (path={0})."));


				if (tc.isEntryEnabled()) Tr.exit(tc, "renameURIVirtualHost", r);
				throw r;
			}

			if (currentVirtualHostName.equals(oldVirtualHostName))
			{
				newURI = newVirtualHostName + currentURIPath;
			}

			// Persist the new servletURIs value. 
			groupURI = newURI;
			dirty = true;
			ejbStore();

			// Update all Servlets under this SerlvetGroup.
			TypeHome typeHome = (TypeHome) getHome("TypeHome");
			Type servletType = typeHome.findByImplClass("com.ibm.ejs.sm.beans.ServletBean", true);

			Enumeration servlets = listContainedObjects(servletType);
			while (servlets.hasMoreElements())
			{
				Servlet servlet = (Servlet)javax.rmi.PortableRemoteObject.narrow(servlets.nextElement(), 
																																				 Servlet.class);

				servlet.renameURIsVirtualHost(oldVirtualHostName, newVirtualHostName);
			}
		}
		catch (FinderException e)
		{
			RemoteException r = new RemoteException((nls.getString("segb.renameurivh.servlet.type.exception",
																														 "Exception occurred attempting to find ServletBean Type.")),
																							e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "renameURIVirtualHost", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "renameURIVirtualHost");
	}

	/*
	 * getServletNames - Returns a Vector of the names of all the Servlets 
	 *                   associated with this ServletGroup.
	 */
	public Vector getServletNames()
	throws RemoteException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "getServletNames");

		Long primaryKey = (Long) ec.getPrimaryKey();

		try
		{
			Vector servletNames = new Vector();

			// Get all contained Servlets for this ServletGroup.
			TypeHome typeHome = (TypeHome) getHome("TypeHome");
			Type servletType = typeHome.findByImplClass("com.ibm.ejs.sm.beans.ServletBean", true);

			Enumeration servlets = listContainedObjects(servletType);
			while (servlets.hasMoreElements())
			{
				Servlet servlet = (Servlet)javax.rmi.PortableRemoteObject.narrow(servlets.nextElement(), 
																																				 Servlet.class);
				ServletAttributes servletAttrs = new ServletAttributes();
				servletAttrs = (ServletAttributes) (servlet.getAttributes(servletAttrs));
				String servletName = servletAttrs.getName();
				servletNames.addElement(servletName);
			}

			if (tc.isEntryEnabled()) Tr.exit(tc, "getServletNames");
			return servletNames;
		}
		catch (OpException e)
		{
			RemoteException r =  new RemoteException((nls.getFormattedMessage("segb.servlet.exception",
																																				new Object[] { primaryKey},
																																				"Exception occurred attempting to get Servlet instances for the ServletGroup (id={0}).")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "getServletNames", r);
			throw r;
		}
		catch (RemoteException e)
		{
			RemoteException r =  new RemoteException((nls.getFormattedMessage("segb.servlet.exception",
																																				new Object[] { primaryKey},
																																				"Exception occurred attempting to get Servlet instances for the ServletGroup (id={0}).")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "getServletNames", r);
			throw r;
		}
		catch (FinderException e)
		{
			RemoteException r =  new RemoteException((nls.getString("segb.servlet.find.exception",
																															"Exception occurred attempting to find ServletBean Type instance.")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "getServletNames", r);
			throw r;
		}
		catch (AttributeNotSetException e)
		{
			RemoteException r =  new RemoteException((nls.getString("segb.attribute.not.set2.exception",
																															"Servlet name attribute was not set.")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "getServletNames", r);
			throw r;
		}
	}

	/*
	 * Helper Methods
	 */

	/*
	 * initializeNLS -- Creates and initializes the NLS object.
	 */
	protected static void initializeNLS(String propertiesFile)
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "initializeNLS");

		synchronized (classLock) 
		{
			if (nls != null)
			{
				if (tc.isEntryEnabled()) Tr.exit(tc, "initializeNLS");
				return;
			}
			else
			{
				nls = new NLS(propertiesFile);
			}
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "initializeNLS");
	}

	/* 
	 * updateServletURIs --  Update all the URIs for all the Servlets under this ServletGroup
	 *                       with the new input root URI path
    * @88536 modified - see old code below
	 */
   private void updateServletURIs(String oldRootURI, String newRootURI, boolean isCloneInstance)
   throws RemoteException, InvalidURINameException
   {
      if (tc.isEntryEnabled()) Tr.entry(tc, "updateServletURIs");

      Long primaryKey = (Long) ec.getPrimaryKey();
      int firstSlash ;
      String oldURIPath ;


      //******************
      if (isCloneInstance) {

         try {
            // Grab the oldRoot URI without the virtualHost.
            firstSlash = oldRootURI.indexOf("/");
            oldURIPath = null;
            if (firstSlash >= 0) {
               oldURIPath = oldRootURI.substring(firstSlash);
            } else {
               oldURIPath = oldRootURI;
            }

            ModelHome modelHome = (ModelHome) RepositoryObjectImpl.getHome("ModelHome");

            Model model = modelHome.findByName(getModelName(), true);

            Enumeration models = model.listContainedObjects();

            while (models.hasMoreElements()) {
               Model containedModels = (Model)javax.rmi.PortableRemoteObject.narrow(models.nextElement(),Model.class);
               Attributes mattrs= containedModels.getAttributes(new ModelAttributes());
               Attributes sattrs=((ModelAttributes)mattrs).getAttributes();


               Vector uris = ((ServletAttributes)sattrs).getURIPaths();

               Vector newURIs = new Vector();

               // For each path, strip the old rootURI off and append the 
               // new rootURI on and add it to the Vector of new Servlet URIs.
               int numURIs = uris.size();
               for (int i = 0; i < numURIs; i++) {
                  String currentPath = (String)(uris.elementAt(i));
                  String minusRootPath = null;
                  String newURIPath = null;

                  // If the old root URI is just "/", then don't strip it off!
                  if (oldURIPath.equals("/")) {
                     minusRootPath = currentPath.substring((oldRootURI.length())-1);
                  } else {
                     minusRootPath = currentPath.substring(oldRootURI.length());
                  }

                  newURIPath = newRootURI + minusRootPath;

                  newURIs.addElement(newURIPath);
               }

               ((ServletAttributes)sattrs).setURIPaths(newURIs);

               ((ModelAttributes)mattrs).setAttributes(sattrs);

               containedModels.setAttributes(mattrs);
            }
         } catch (Exception e) {
            RemoteException r = new RemoteException("", e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs");
			throw r;

         }
      } else {

         //******************

         try {
            // Grab the oldRoot URI without the virtualHost.
            firstSlash = oldRootURI.indexOf("/");
            oldURIPath = null;
            if (firstSlash >= 0) {
               oldURIPath = oldRootURI.substring(firstSlash);
            } else {
               oldURIPath = oldRootURI;
            }

            // Get all contained Servlets for this ServletGroup.
            TypeHome typeHome = (TypeHome) getHome("TypeHome");
            Type servletType = typeHome.findByImplClass("com.ibm.ejs.sm.beans.ServletBean", true);

            Enumeration servlets = listContainedObjects(servletType);
            while (servlets.hasMoreElements()) {
               Servlet servlet = (Servlet)javax.rmi.PortableRemoteObject.narrow(servlets.nextElement(), 
                                                                                Servlet.class);
               ServletAttributes servletAttrs = new ServletAttributes();
               servletAttrs = (ServletAttributes) (servlet.getAttributes(servletAttrs)); 
               Vector uris = servletAttrs.getURIPaths();
               Vector newURIs = new Vector();

               // For each path, strip the old rootURI off and append the 
               // new rootURI on and add it to the Vector of new Servlet URIs.
               int numURIs = uris.size();
               for (int i = 0; i < numURIs; i++) {
                  String currentPath = (String)(uris.elementAt(i));
                  String minusRootPath = null;
                  String newURIPath = null;

                  // If the old root URI is just "/", then don't strip it off!
                  if (oldURIPath.equals("/")) {
                     minusRootPath = currentPath.substring((oldRootURI.length())-1);
                  } else {
                     minusRootPath = currentPath.substring(oldRootURI.length());
                  }

                  newURIPath = newRootURI + minusRootPath;

                  newURIs.addElement(newURIPath);
               }

               // Update the URIs for the current servlet.
               servlet.setURIs(newURIs, isCloneInstance);
            }
         } catch (OpException e) {
            RemoteException r =  new RemoteException((nls.getFormattedMessage("segb.servlet.exception",
                                                                              new Object[] { primaryKey},
                                                                              "Exception occurred attempting to get  Servlet instances for the ServletGroup (id={0}).")), 
                                                     e);
            if (tc.isEntryEnabled()) Tr.exit(tc, "getServletNames", r);
            throw r;
         } catch (RemoteException e) {
            RemoteException r =  new RemoteException((nls.getFormattedMessage("segb.servlet.exception",
                                                                              new Object[] { primaryKey},
                                                                              "Exception occurred attempting to get Servlet instances for the ServletGroup (id={0}).")), 
                                                     e);
            if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs", r);
            throw r;
         } catch (FinderException e) {
            RemoteException r =  new RemoteException((nls.getString("segb.servlet.find.exception",
                                                                    "Exception occurred attempting to find ServletBean Type instance.")), 
                                                     e);
            if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs", r);
            throw r;
         } catch (AttributeNotSetException e) {
            RemoteException r =  new RemoteException((nls.getString("segb.attribute.not.set3.exception",
                                                                    "Servlet uriPaths attribute was not set.")), 
                                                     e);
            if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs", r);
            throw r;
         }
      }

      if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs");
   }
}
   
   /*
	private void updateServletURIs(String oldRootURI, String newRootURI, boolean isCloneInstance)
	throws RemoteException, InvalidURINameException
	{
		if (tc.isEntryEnabled()) Tr.entry(tc, "updateServletURIs");

		Long primaryKey = (Long) ec.getPrimaryKey();

		try
		{
			// Grab the oldRoot URI without the virtualHost.
			int firstSlash = oldRootURI.indexOf("/");
			String oldURIPath = null;
			if (firstSlash >= 0)
			{
				oldURIPath = oldRootURI.substring(firstSlash);
			}
			else
			{
				oldURIPath = oldRootURI;
			}

			// Get all contained Servlets for this ServletGroup.
			TypeHome typeHome = (TypeHome) getHome("TypeHome");
			Type servletType = typeHome.findByImplClass("com.ibm.ejs.sm.beans.ServletBean", true);

			Enumeration servlets = listContainedObjects(servletType);
			while (servlets.hasMoreElements())
			{
				Servlet servlet = (Servlet)javax.rmi.PortableRemoteObject.narrow(servlets.nextElement(), 
																																				 Servlet.class);
				ServletAttributes servletAttrs = new ServletAttributes();
				servletAttrs = (ServletAttributes) (servlet.getAttributes(servletAttrs)); 
				Vector uris = servletAttrs.getURIPaths();
				Vector newURIs = new Vector();

				// For each path, strip the old rootURI off and append the 
				// new rootURI on and add it to the Vector of new Servlet URIs.
				int numURIs = uris.size();
				for (int i = 0; i < numURIs; i++)
				{
					String currentPath = (String)(uris.elementAt(i));
					String minusRootPath = null;
					String newURIPath = null;

					// If the old root URI is just "/", then don't strip it off!
					if (oldURIPath.equals("/"))
					{
						minusRootPath = currentPath.substring((oldRootURI.length())-1);
					}
					else
					{
						minusRootPath = currentPath.substring(oldRootURI.length());
					}

					newURIPath = newRootURI + minusRootPath;

					newURIs.addElement(newURIPath);
				}

				// Update the URIs for the current servlet.
				servlet.setURIs(newURIs, isCloneInstance);
			}
		}
		catch (OpException e)
		{
			RemoteException r =  new RemoteException((nls.getFormattedMessage("segb.servlet.exception",
																																				new Object[] { primaryKey},
																																				"Exception occurred attempting to get  Servlet instances for the ServletGroup (id={0}).")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "getServletNames", r);
			throw r;
		}
		catch (RemoteException e)
		{
			RemoteException r =  new RemoteException((nls.getFormattedMessage("segb.servlet.exception",
																																				new Object[] { primaryKey},
																																				"Exception occurred attempting to get Servlet instances for the ServletGroup (id={0}).")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs", r);
			throw r;
		}
		catch (FinderException e)
		{
			RemoteException r =  new RemoteException((nls.getString("segb.servlet.find.exception",
																															"Exception occurred attempting to find ServletBean Type instance.")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs", r);
			throw r;
		}
		catch (AttributeNotSetException e)
		{
			RemoteException r =  new RemoteException((nls.getString("segb.attribute.not.set3.exception",
																															"Servlet uriPaths attribute was not set.")), 
																							 e);
			if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs", r);
			throw r;
		}

		if (tc.isEntryEnabled()) Tr.exit(tc, "updateServletURIs");
	}
}
*/
