/*
 *
 * COMPONENT_NAME: <FILL_ME_IN>
 *
 * ORIGINS: Transarc Corp.
 *
 * (C) COPYRIGHT Transarc Corp. 1998
 * All Rights Reserved
 * Licensed Materials - Property of Transarc
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with Transarc Corp
 */

package com.ibm.ejs.sm.client.ui.editor;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.FinderException;

import com.sun.java.swing.*;     
import com.sun.java.swing.border.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.tree.*;
import com.sun.java.swing.text.*;
import com.sun.java.swing.plaf.*;
import com.sun.java.swing.plaf.basic.*;

import com.ibm.ejs.sm.client.ui.ComponentManager;
import com.ibm.ejs.sm.beans.*;
import com.ibm.ejs.sm.client.ui.*;
import com.ibm.ejs.ras.*;
import com.ibm.ejs.sm.beans.*;
import com.ibm.ejs.sm.client.*;

import com.ibm.ejs.sm.client.command.*;
import com.ibm.ejs.sm.client.util.Utils;
import com.ibm.ejs.sm.client.util.OrbHelper;
import com.ibm.ejs.sm.exception.*;

public class ServletPropertySheet extends LiveROPropertySheet implements ActionListener
{

   private static TraceComponent tc = Tr.register(ServletPropertySheet.class);


   private TypeInstance servlet;
   private TypeObject   ejbType;
   private ServletHome servletHome;
   private ServletAttributes cattrs;
   private ModelAttributes mattrs = null;
   private ServletAttributes initialAttributes;
   private ActionListener myActionListener;
   private RepositoryObjectProxy ro[];
   private TypeObject to;


   private GridBagLayout gbl;
   private GridBagConstraints gbc;


   private JLabel groupNameL;

   private JComboBox groupListComboBox;

   private JPanel buttonPanel;
   private JButton setButton;
   private JButton cancelButton;

   private JPanel generalPropsPanel;
   private JPanel advancedPropsPanel;
   private JPanel infoPanel;
   private JPanel startupPanel;

   protected JLabel enabledL; 
   protected JLabel enabledStringL;

   private JLabel descriptionL ;
   private JTextField descriptionT;
   private JLabel codeL ;
   private JTextField codeT;

   private JLabel uriStringL;
   private JTextField uriStringTF;

   private JList uriList;
   private JScrollPane uriListScroller ;

   private JPanel uriPanel;
   private JPanel uriButtonPanel;
   private JButton addButton;
   private JButton editButton;
   private JButton deleteButton;

   private URI groupURI;

   private String[] boolStrings;

   // advanced
   private JLabel initParmsL;
   private PropertyEditor initParmsPE;
   private JLabel  debugModeL;
   private JComboBox debugModeComboBox;
   private JLabel  loadAtStartupL;
   private JComboBox loadAtStartupComboBox;

   private JLabel codeActiveL ;
   private JLabel codeActiveT;
   private JLabel uriListActiveL;
   private JList uriListActive;
   private JScrollPane uriListScrollerActive ;
   private JLabel initParmsActiveL;
   private PropertyEditor initParmsActivePE;
   private JLabel  debugModeActiveL;
   private JLabel  debugModeActiveT;
   private JLabel  loadAtStartupActiveL;
   private JLabel  loadAtStartupActiveT;

   String name;
   String servletClassName;

   boolean dirtyURIs = false;
   Vector oldURIs;

   private static String servletTypeName = "com.ibm.ejs.sm.beans.Servlet";

   //    Dimension minimumSize   = new Dimension(400,400);
   Dimension preferredSize = new Dimension(400,400);

   private final static int FIELD_SIZE = 15;


   public ServletPropertySheet(Integer mode, TypeInstance ejb)
   {
      this(mode.intValue(), ejb);
   }
   public ServletPropertySheet(Integer mode, TypeObject ejbType)
   {
      this(mode.intValue(), ejbType);
   }

   public ServletPropertySheet(int mode)
   {
      this(mode, TypeObject.findTypeObject(servletTypeName));
   }

   public ServletPropertySheet(int mode, TypeObject ejbType)
   {
      super(mode,ejbType);
      to = TypeObject.findTypeObject(servletTypeName);
      Tr.entry(tc, "ServletPropertySheet, mode: " + mode + "  type: " + ejbType);
      this.ejbType = ejbType;
      notSetString = labelNLS.getString("propertySheet.attributeNotSet",
                                        "Attribute not set");
      postInit();
      Tr.exit(tc, "ServletPropertySheet");
   }

   public ServletPropertySheet(int mode, TypeInstance servlet)
   {
      super(mode,servlet);
      to = TypeObject.findTypeObject(servletTypeName);
      if (modeBits.get(MODEL_MODE) && (servlet.getREJB() instanceof Model))
      {
         try
         {
            mattrs = (ModelAttributes) servlet.getAttributes();
            liveAttributes = cattrs = (ServletAttributes)
                             mattrs.getAttributes();
            Tr.event(tc, "ServletAttributes initialized", cattrs);
            this.servlet = servlet;
            ejbType = TypeObject.findTypeObject(servletTypeName);
         }
         catch (Exception e)
         {
            Tr.warning(tc, "Error.in.getting.model.servlet.attrs", e);
         }
      }
      else
      {
         this.servlet = servlet;
         this.ejbType      = servlet.getTypeObject();
      }
      notSetString = labelNLS.getString("propertySheet.attributeNotSet",
                                        "Attribute not set");
      postInit();
      Tr.exit(tc, "ServletPropertySheet");
   }

   private void setInitialAttributes(ServletAttributes cattrs)
   {
      initialAttributes = cattrs;
   }

   protected void postInit()
   {
      Tr.entry(tc, "postInit");
      if (cattrs == null)
      {
         liveAttributes = cattrs = (ServletAttributes)getInitialAttributes(servlet,ejbType);
         Tr.event(tc, "ServletEngine initialized", cattrs);
      }
      setInitialAttributes(cattrs);
      createComponents();
      super.postInit();
      Tr.exit(tc, "postInit");
   }

   protected  void createComponents()
   {
      Tr.entry(tc, "createComponents");
      super.createComponents();

      /*
      ** setup Props
        ** enabled
      ** Pick Connecting Servlet App
      ** Name (done by BasePropSheet)x
      ** Description
      ** DocRoot
      ** Classpath
      ** Pick URI
      */
      enabledL = getLabel(labelNLS.getString("any.enabled.label","Enabled"));
      enabledStringL = new JLabel();

      groupNameL = getRequiredLabel(labelNLS.getString("servlet.appgroupname.label", "Web Application"));
      groupListComboBox = new JComboBox();

      descriptionL = getLabel(labelNLS.getString("servlet.servletdescription.label", "Description"));
      descriptionT = new JTextField(FIELD_SIZE);

      codeL = getRequiredLabel(labelNLS.getString("servlet.servletcode.label", "Servlet Class Name"));
      codeT = new JTextField(FIELD_SIZE);
      codeT.addKeyListener(new KeyAdapter()
                           {
                              public void keyReleased(KeyEvent e)
                              {
                                 firePropertyChange("servletcode", "", codeT.getText());
                              }
                           });


      uriStringL =  getRequiredLabel(labelNLS.getString("servlet.uristring.label", "Edit Full Servlet Web Path"));
      uriStringTF =  new JTextField(FIELD_SIZE);
      uriStringTF.addKeyListener(new KeyAdapter()
                                 {
                                    public void keyReleased(KeyEvent e)
                                    {
                                       firePropertyChange("servleturi", "", uriStringTF.getText());
                                    }
                                 });


      uriPanel = new JPanel();
      uriPanel.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(),labelNLS.getString("servlet.uri.list","Servlet Web Path List")));

      uriList = new JList();
      uriList.setVisibleRowCount(3);
      uriList.setModel(new DefaultListModel());
      uriList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      uriListScroller = new JScrollPane(uriList);
      final int opMode = mode;
      uriList.addMouseListener(new MouseAdapter()
                               {
                                  public void mouseClicked(MouseEvent e)
                                  {
                                     if (e.getClickCount() == 2 && opMode != BasePropertySheet.READ_ONLY_MODE)
                                     {
                                        JList list = (JList) e.getSource();
                                        DefaultListModel model = (DefaultListModel)list.getModel();
                                        int index = list.locationToIndex(e.getPoint());
                                        if (index >= 0)
                                        {
                                           String itemString =  model.getElementAt(index).toString();
                                           if (groupListComboBox.getSelectedIndex()<0)
                                           {
                                              EJSConsoleDialog.showInfo(labelNLS.getString("servlet.group.pleaseselectwebapp", "Please Select a Web App for this Servlet"));
                                           }
                                           else
                                           {
                                              new SRTreePanel(index);
                                           }
                                        }
                                     }
                                  }
                               });

      addButton = new JButton()
      {
         public Dimension getPreferredSize()
         {
            return UIConstants.defaultButtonSize;
         }
         public Dimension getMaximumSize()
         {
            return UIConstants.defaultButtonSize;
         }
      };
      addButton.setText(labelNLS.getString("servlet.button.add","Add"));
      addButton.setActionCommand("Add");
      addButton.addActionListener(this);

      editButton = new JButton()
      {
         public Dimension getPreferredSize()
         {
            return UIConstants.defaultButtonSize;
         }
         public Dimension getMaximumSize()
         {
            return UIConstants.defaultButtonSize;
         }
      };
      editButton.setText(labelNLS.getString("servlet.button.edit","Edit"));
      editButton.setActionCommand("Edit");
      editButton.addActionListener(this);

      deleteButton = new JButton()
      {
         public Dimension getPreferredSize()
         {
            return UIConstants.defaultButtonSize;
         }
         public Dimension getMaximumSize()
         {
            return UIConstants.defaultButtonSize;
         }
      };
      deleteButton.setText(labelNLS.getString("servlet.button.delete","Delete"));
      deleteButton.setActionCommand("Delete");
      deleteButton.addActionListener(this);

      // Control Button Panel
      uriButtonPanel = new JPanel();
      uriButtonPanel.add(addButton);
      uriButtonPanel.add(editButton);        
      uriButtonPanel.add(deleteButton);    

      uriPanel.setLayout(new BorderLayout());
      uriPanel.add(uriListScroller,BorderLayout.NORTH);
      uriPanel.add(uriButtonPanel,BorderLayout.SOUTH);


      codeActiveL = getLabel(labelNLS.getString("servlet.activeservletcode.label", "Servlet Class Name in use"));
      codeActiveT = new JLabel();
      codeActiveT.setForeground(UIConstants.activeAttrColor);

      uriListActiveL = getLabel(labelNLS.getString("servlet.activeuri.list","Servlet Web Paths List in use"));
      uriListActive = new JList();
      uriListActive.setModel(new DefaultListModel());
      uriListActive.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      uriListScrollerActive = new JScrollPane(uriListActive);
      uriListActive.setVisibleRowCount(3);
      /*
      ** create Advanced Props
      ** ErrorPage
      ** Init Params
      ** FilterList
        ** load at Startup
      */

      initParmsL = getLabel(labelNLS.getString("servlet.servletinitparms.label", "Init Parameters"));
      Vector v = new Vector();
      v.addElement(labelNLS.getString("servlet.servletinitparms.name", "Init Parm Name"));
      v.addElement(labelNLS.getString("servlet.servletinitparms.value", "Init Parm Value"));
      Properties p = new Properties();
      initParmsPE = new PropertyEditor(v,p);

      loadAtStartupL = getLabel(labelNLS.getString("servlet.servletloadatstartup.label", "Load at Startup"));

      boolStrings = new String[2];
      boolStrings[0]=labelNLS.getString("any.true.label", "True");
      boolStrings[1]=labelNLS.getString("any.false.label","False");

      loadAtStartupComboBox = new JComboBox(boolStrings);

      debugModeL = getLabel(labelNLS.getString("servlet.servletdebugmode.label", "Debug Mode"));
      debugModeComboBox = new JComboBox(boolStrings);

      initParmsActiveL = getLabel(labelNLS.getString("servlet.activeservletinitparms.label", "Init Parameters in use")); 
      v = new Vector();
      v.addElement(labelNLS.getString("servlet.servletinitparms.name", "Init Parm Name"));
      v.addElement(labelNLS.getString("servlet.servletinitparms.value", "Init Parm Value"));
      p = new Properties();
      initParmsActivePE = new ImmutablePropertyEditor(v,p);

      loadAtStartupActiveL = getLabel(labelNLS.getString("servlet.activeservletloadatstartup.label", "Load at Startup in use"));
      loadAtStartupActiveT = new JLabel();
      loadAtStartupActiveT.setForeground(UIConstants.activeAttrColor);

      debugModeActiveL = getLabel(labelNLS.getString("servlet.activeservletdebugmode.label", "Debug Mode in use"));
      debugModeActiveT = new JLabel();
      debugModeActiveT.setForeground(UIConstants.activeAttrColor);

      reflectAttributes();
      layoutPanel();
      Tr.exit(tc, "createComponents");
   }       

   private void layoutPanel()
   {
      Tr.entry(tc, "layoutPanel");
      clearPanel();
      setUpGUI();
      validate();
      repaint();
      Tr.exit(tc, "layoutPanel");
   }

   public void setURITextField(String uriT)
   {
      uriStringTF.setText(uriT);
   }

   public PropPanel getGeneralPanel()
   {
      Tr.entry(tc, "getGeneralPanel");
      PropPanel generalPropsPanel = super.getGeneralPanel();
      gbc = generalPropsPanel.gbc;
      gridy = gbc.gridy;
      gridy++;

      /**
       * setup General
       */
      if (modeBits.get(EDIT_MODE) || 
          modeBits.get(CONTENT_PANE_MODE))
      {
         gbc.gridx=0;gbc.gridy=gridy;
         generalPropsPanel.add(enabledL,gbc);
         gbc.gridx=1;
         generalPropsPanel.add(enabledStringL,gbc);
         gridy++;
      }

      if ((!modeBits.get(PropertySheet.EDIT_DEFAULTS_MODE)) && (!modeBits.get(PropertySheet.STAND_ALONE_MODE)) &&  (!modeBits.get(MODEL_MODE)))
      {
         gbc.gridx=0;gbc.gridy=gridy;
         generalPropsPanel.add(groupNameL,gbc);
         gbc.gridx=1;
         generalPropsPanel.add(groupListComboBox,gbc);
      }

      gridy++;
      gbc.gridx=0;gbc.gridy=gridy;
      generalPropsPanel.add(descriptionL,gbc);
      gbc.gridx=1;
      generalPropsPanel.add(descriptionT,gbc);

      gridy++;
      gbc.gridx=0;gbc.gridy=gridy;
      generalPropsPanel.add(codeL,gbc);
      gbc.gridx=1;
      gbc.weightx=1;
      generalPropsPanel.add(codeT,gbc);
      gbc.weightx=0;

      if (modeBits.get(EDIT_MODE) || 
          modeBits.get(CONTENT_PANE_MODE))
      {
         gridy++;
         gbc.gridx=0;gbc.gridy=gridy;
         generalPropsPanel.add(codeActiveL,gbc);
         gbc.gridx=1;
         generalPropsPanel.add(codeActiveT,gbc);
      }

      if (!(modeBits.get(EDIT_DEFAULTS_MODE)))
      {
         gridy++;
         gbc.gridx=0;gbc.gridy=gridy;
         gbc.gridwidth=2;
         generalPropsPanel.add(uriPanel,gbc);
         gbc.gridwidth=1;
      }

      if (modeBits.get(EDIT_MODE) || 
          modeBits.get(CONTENT_PANE_MODE))
      {
         gridy++;
         gbc.gridx=0;gbc.gridy=gridy;
         generalPropsPanel.add(uriListActiveL,gbc);
         gridy++;
         gbc.gridwidth=2;
         gbc.gridx=0;gbc.gridy=gridy;
         generalPropsPanel.add(uriListScrollerActive,gbc);
         gbc.gridwidth=1;
      }
      Tr.exit(tc, "getGeneralPanel");
      return generalPropsPanel;
   }

   public PropPanel getAdvancedPanel()
   {
      Tr.entry(tc, "getAdvancedPanel");
      PropPanel advancedPropsPanel = new PropPanel();
      GridBagConstraints gbc = advancedPropsPanel.gbc = new GridBagConstraints();
      GridBagLayout gbl = new GridBagLayout();
      gbc.insets = new Insets(4,4,4,4);
      /**
       * setup advanced props
       */
      gbc.fill = GridBagConstraints.HORIZONTAL;
      gbc.anchor = GridBagConstraints.NORTH;
      gridy = 0 ;
      advancedPropsPanel.setLayout(gbl);

      gridy++;
      gbc.gridx=0;gbc.gridy=gridy;
      advancedPropsPanel.add(initParmsL,gbc);
      gbc.gridx=1;
      JScrollPane scrollPane = new JScrollPane(initParmsPE);
      advancedPropsPanel.add(scrollPane,gbc);


      if (modeBits.get(EDIT_MODE) || 
          modeBits.get(CONTENT_PANE_MODE))
      {
         gridy++;
         gbc.gridx=0;gbc.gridy=gridy;
         advancedPropsPanel.add(initParmsActiveL,gbc);
         gbc.gridx=1;
         scrollPane = new JScrollPane(initParmsActivePE);
         advancedPropsPanel.add(scrollPane,gbc);
      }

      gridy++;
      gbc.gridx=0;gbc.gridy=gridy;
      advancedPropsPanel.add(debugModeL,gbc);
      gbc.gridx=1;
      advancedPropsPanel.add(debugModeComboBox,gbc);

      if (modeBits.get(EDIT_MODE) || 
          modeBits.get(CONTENT_PANE_MODE))
      {
         gridy++;
         gbc.gridx=0;gbc.gridy=gridy;
         advancedPropsPanel.add(debugModeActiveL,gbc);
         gbc.gridx=1;
         advancedPropsPanel.add(debugModeActiveT,gbc);
      }

      gridy++;
      gbc.gridx=0;gbc.gridy=gridy;
      advancedPropsPanel.add(loadAtStartupL,gbc);
      gbc.gridx=1;
      gbc.weightx=1;
      advancedPropsPanel.add(loadAtStartupComboBox,gbc);
      gbc.weightx=0;

      if (modeBits.get(EDIT_MODE) || 
          modeBits.get(CONTENT_PANE_MODE))
      {
         gridy++;
         gbc.gridx=0;gbc.gridy=gridy;
         advancedPropsPanel.add(loadAtStartupActiveL,gbc);
         gbc.gridx=1;
         advancedPropsPanel.add(loadAtStartupActiveT,gbc);
      }
      Tr.exit(tc, "getAdvancedPanel");
      return advancedPropsPanel;
   }

   protected JPanel getContainmentInfoPanel()
   {
      Tr.entry(tc, "getContainmentInfoPanel");
      if (modeBits.get(MODEL_MODE) && modeBits.get(CREATE_MODE))
      {
         if (getSelectedServletGroup() != null)
         {
            Tr.exit(tc, "getContainmentInfoPanel");
            return null;
         }
         Tr.exit(tc, "getContainmentInfoPanel");
         return super.getContainmentTreePanel(TypeObject.findTypeObject(modelTypeName));
      }
      else
      {
         Tr.exit(tc, "getContainmentInfoPanel");
         return null;
      }
   }

   private RepositoryObjectProxy getSelectedServletGroup()
   {
      Tr.entry(tc, "getSelectedServletGroup");
      if (proxy.getREJB() instanceof ServletGroup)
      {
         Tr.debug(tc, "getSelectedServletGroup",proxy);
         Tr.exit(tc, "getSelectedServletGroup",proxy);
         return proxy;
      }
      else Tr.debug(tc, "Cannot find valid server choice.");
      Tr.exit(tc, "getSelectedServletGroup");
      return null;
   }

   public void modifyAttributes()
   {
      Tr.entry(tc, "modifyAttributes");
      servlet = (TypeInstance)SelectionCoordinator.getSelectedProxy();
      if (modeBits.get(MODEL_MODE))
      {
         super.modifyAttributes();
      }
      else
      {
         ServletAttributes newAttrs = (ServletAttributes)getAttributes();
         if (newAttrs == null) return;
         if (dirtyURIs)
         {
            Vector v = getOldURIs(initialAttributes);
            MyCommandWatcher myCmdwatcher = new MyCommandWatcher(new ModifyAttributesCommand(servlet, initialAttributes, newAttrs),v,newAttrs);

         }
         else
         {
            if ((modeBits.get(EDIT_MODE)) || (modeBits.get(CONTENT_PANE_MODE)))
            {
               new ModifyAttributesCommand(servlet, initialAttributes, newAttrs);
            }
            else
            {
               new ModifyAttributesCommand(to, newAttrs);
            }
         }
      }
      Tr.exit(tc, "modifyAttributes");
   }

   private Vector getOldURIs(ServletAttributes attrs)
   {
      Tr.exit(tc, "getOldURIs");
      Vector v = oldURIs;
      Vector uriV = new Vector();
      Enumeration e = v.elements();
      URIHome uriHome = null;
      try
      {
         uriHome = (URIHome)RepositoryClient.getHome("URIHome");
      }
      catch (RepositoryOpException rop)
      {
         new ErrorDialog(rop);
         return null;
      }

      String uriName;
      while (e.hasMoreElements())
      {
         uriName = e.nextElement().toString();
         try
         {
            uriV.addElement(uriHome.findByName(uriName,false));
         }
         catch (FinderException fe)
         {
            new ErrorDialog(fe);
            Tr.exit(tc, "getOldURIs");
            return uriV;
         }
         catch (RemoteException re)
         {
            new ErrorDialog(re);
            Tr.exit(tc, "getOldURIs");
            return uriV;
         }
      }

      return  uriV;
   }

   public class MyCommandWatcher implements CommandListener
   {
      private AbstractCommand cmd;
      private Vector oldURIs;
      private ServletAttributes sattrs;

      MyCommandWatcher(AbstractCommand cmd,Vector v, ServletAttributes a)
      {
         CommandManager.addCommandListener(this);
         this.cmd = cmd;
         oldURIs = v;
         sattrs = a;
      }

      public void commandComplete(CommandEvent evt)
      {
         Tr.entry(tc,"commandComplete",evt);         
         if (evt.getCommand().equals(cmd))
         {
            if (evt.getStatus() == CommandStatus.SUCCESS)
            {
               if (dirtyURIs)
               {
                  deleteURIs(oldURIs);
                  updateURIs(sattrs);
                  dirtyURIs = false;
               }
            }
            CommandManager.removeCommandListener(this);
         }
         Tr.exit(tc,"commandComplete");          
      }

      public void commandStarted(CommandEvent evt)
      {
      }

      public void commandProgress(CommandEvent evt)
      {
      }
   }

   public void deleteURIs(Vector v)
   {
      Tr.entry(tc, "deleteURIs");
      RepositoryModel rmodel = RepositoryClient.sharedInstance().getRepositoryModel();
      try
      {
         if (v  == null) return;
         InstanceTree it = rmodel.getITree();
         Enumeration e = v.elements();
         URI uri;
         TypeInstance ti;
         while (e.hasMoreElements() )
         {
            uri = (URI)e.nextElement();
            ti = it.findInstance(uri);
            if (it != null)
            {
               it.remove(ti);
            }
         }
      }
      catch (ClientException ce)
      {
         new ErrorDialog(ce);
      }
      Tr.exit(tc, "deleteURIs");
   }

   public void updateURIs(ServletAttributes a)
   {
      Tr.entry(tc, "updateURIs");
      RepositoryModel rmodel = RepositoryClient.sharedInstance().getRepositoryModel();
      InstanceTree it = rmodel.getITree();
      TypeInstance ti;
      Vector uriVector = null;
      try
      {
         uriVector = a.getURIPaths();
      }
      catch (AttributeNotSetException anse)
      {
         new ErrorDialog(anse);
         Tr.exit(tc, "updateURIs");
         return;
      }
      if ((uriVector == null) || ( uriVector.size() <=0))
      {
         Tr.exit(tc, "updateURIs no URIs found");
         return;
      }
      String uriName = uriVector.elementAt(0).toString();
      String vhName = uriName.substring(0,uriName.indexOf("/"));
      RepositoryObjectProxy rop[] = Utils.getAllObjectsOfType("com.ibm.ejs.sm.beans.VirtualHost");
      RepositoryObjectProxy vhp=null;
      TypeInstance vhi;
      VirtualHost vh = null;
      for (int i = 0;i<rop.length;i++)
      {
         if (rop[i].getName().equals(vhName))
         {
            vhp = rop[i];
            vh = (VirtualHost)vhp.getREJB();
            break;
         }
      }
      if ((vh == null) || (vhp == null)) return;

      try
      {
         URIHome uriHome = null;
         uriHome = (URIHome)RepositoryClient.getHome("URIHome");
         Enumeration e = uriVector.elements();
         String uriS= null;
         URI uri;
         vhi = (TypeInstance) vhp;
         while (e.hasMoreElements())
         {
            uriS= e.nextElement().toString();
            uri = uriHome.findByName(uriS,false);                   
            if (uri != null)
            {
               ti = it.findInstance(uri);
               if (ti == null)
               {
                  rmodel.insert(uri,vhi);
               }
            }
         }
         oldURIs = uriVector;
      }
      catch (ClientException ce)
      {
         new ErrorDialog(ce);
         Tr.exit(tc, "updateURIs");
         return;
      }
      catch (FinderException fe)
      {
         new ErrorDialog(fe);
         Tr.exit(tc, "updateURIs");
         return;
      }
      catch (RepositoryOpException roe)
      {
         new ErrorDialog(roe);
         Tr.exit(tc, "updateURIs");
         return;
      }
      catch (RemoteException re)
      {
         new ErrorDialog(re);
         Tr.exit(tc, "updateURIs");
         return;
      }
   }

   public void modify(Attributes initialAttrs)
   {
      Tr.entry(tc, "modifyAttributes",initialAttrs);
      servlet = (TypeInstance)SelectionCoordinator.getSelectedProxy();
      ServletAttributes newAttrs = (ServletAttributes)getAttributes();
      if (newAttrs == null) return;
      if (modeBits.get(EDIT_MODE) ||
          modeBits.get(CONTENT_PANE_MODE))
      {
         new ModifyAttributesCommand(servlet, initialAttributes, newAttrs);
      }
      else
      {
         new ModifyAttributesCommand(ejbType, newAttrs);
      }
      Tr.exit(tc, "modifyAttributes",newAttrs);
   }

   public void reset(boolean remotely)
   {
      Tr.entry(tc,"reset");
      if (modeBits.get(MODEL_MODE))
      {
         modelPanel.reset(remotely);
         mattrs = (ModelAttributes) resetAttributes(cattrs, remotely);
         try
         {
            liveAttributes = cattrs = 
                             (ServletAttributes) mattrs.getAttributes();
         }
         catch (Exception e)
         {
            Tr.warning(tc, 
                       "Encountered an error in getting model attributes for " +
                       "Servlet {0}", new Object[] {e});
         }
      }
      else
      {
         liveAttributes = cattrs = 
                          (ServletAttributes)resetAttributes(cattrs, remotely);
      }
      reflectAttributes();
      revalidate();
      repaint();
      dirtyURIs = false;
      Tr.exit(tc,"reset");
   }

   public Attributes getAttributes()
   {
      Tr.entry(tc, "getAttributes");
      if (modeBits.get(BasePropertySheet.CREATE_MODE))
         cattrs = new ServletAttributes();
      Tr.exit(tc, "getAttributes");
      return getAttributesFromGUI();
   }

   public void setupDefaults(String name)
   {
      nameTextField.setText(name);
      codeT.setText(name);
   }

   private ServletAttributes getAttributesFromGUI()
   {
      Tr.entry(tc, "getAttributesFromGUI");
      name = getName();
      servletClassName = getServletClassName();
      if (name == null || name.equals(""))
      {
         if (!(modeBits.get(EDIT_DEFAULTS_MODE)|| modeBits.get(MODEL_MODE)))
         {
            showError(labelNLS.getString("servlet.servlet.entername","Please Enter a Valid Name"));
            Tr.debug(tc, "No Servlet name was set");
            return null;
         }
      }
      else if (!modeBits.get(EDIT_DEFAULTS_MODE))
      {
         cattrs.setName(name);
      }

      if (!modeBits.get(EDIT_DEFAULTS_MODE))
      {
         if (servletClassName == null || servletClassName.equals(""))
         {
            showError (labelNLS.getString ("servlet.servlet.enterservletclassname", "Please enter a valid servlet class name"));
            Tr.debug (tc, "No servlet class name was set");
            return null;
         }
         cattrs.setCode(servletClassName);
         cattrs.setURIPaths(getURIs());
      }

      if (modeBits.get(EDIT_DEFAULTS_MODE))
      {
         cattrs = new ServletAttributes();
      }

      cattrs.setDescription(descriptionT.getText());
      cattrs.setInitParams(initParmsPE.getValues());
      cattrs.setDebugMode(debugModeComboBox.getSelectedItem().equals(labelNLS.getString("any.true.label","True"))?1:0);
      cattrs.setLoadAtStartup(loadAtStartupComboBox.getSelectedItem().equals(labelNLS.getString("any.true.label","True")));

      Tr.exit(tc, "getAttributesFromGUI");
      return cattrs;
   }

   public void setGroupSelection(RepositoryObjectProxy rop)
   {
      if (rop != null)
      {
         groupListComboBox.setSelectedItem(rop);
      }
   }
   private Object getGroupSelection()
   {
      if (modeBits.get(MODEL_MODE))
      {
         return SelectionCoordinator.getSelectedProxy();
      }
      else
      {
         return groupListComboBox.getSelectedItem();
      }
   }

   public String getName()
   {
      return nameTextField.getText();
   }

   public String getServletClassName()
   {
      return codeT.getText();
   }

   public ServletGroup getServletGroup()
   {
      RepositoryObjectProxy rop = (RepositoryObjectProxy) getGroupSelection();
      if (rop == null)
      {
         return null;
      }
      ServletGroup group = (ServletGroup) rop.getREJB();
      return group;
   }


   public Servlet getServlet()
   {
      return(Servlet)servlet.getREJB();
   }


   public Vector getURIs()
   {
      Tr.entry(tc, "getURIs");
      Vector uriVector = new Vector();
      DefaultListModel model = (DefaultListModel)uriList.getModel();
      int size = model.getSize();
      Enumeration e ;
      String uriName;
      String uriListEntry;
      boolean matched;
      int slashIndex;
      for (int i=0;i<size;i++)
      {
         // check the vector for dups

         uriListEntry = model.getElementAt(i).toString();
         matched = false;

         e = uriVector.elements();

         while (e.hasMoreElements())
         {
            uriName= e.nextElement().toString();
            if (uriName.equals(uriListEntry))
            {
               matched = true;
               break;
            }
         }
         if (!matched)
         {
            uriVector.addElement(uriListEntry);
         }
      }
      Tr.exit(tc, "getURIs",uriVector);
      return uriVector;
   }


   protected void reflectAttributes()
   {
      Tr.entry(tc, "reflectAttributes");
      super.reflectAttributes();
      if (cattrs == null)
      {
         Tr.event(tc, "Attributes were null");
         return;
      }
      try
      {
         boolean enabled = cattrs.getEnabled();
         if (!enabled)
         {
            enabledStringL.setText(labelNLS.getString("any.false.label","False"));
         }
         else
         {
            enabledStringL.setText(labelNLS.getString("any.true.label","True"));
         }
      }
      catch (AttributeNotSetException anse)
      {
         enabledStringL.setText(labelNLS.getString("any.false.label","False"));
      }
      enabledStringL.setForeground(UIConstants.activeAttrColor);

      try
      {
         descriptionT.setText(cattrs.getDescription());
      }
      catch (AttributeNotSetException anse)
      {
      }


      try
      {
         codeT.setText(cattrs.getCode());
      }
      catch (AttributeNotSetException anse)
      {
      }

      try
      {
         initParmsPE.setValues(cattrs.getInitParams());
      }
      catch (AttributeNotSetException anse)
      {
      }

      try
      {
         if (cattrs.getDebugMode() == 0)
         {
            debugModeComboBox.setSelectedItem(labelNLS.getString("any.false.label", "False"));
         }
         else
         {
            debugModeComboBox.setSelectedItem(labelNLS.getString("any.true.label", "True"));
         }
      }
      catch (AttributeNotSetException anse)
      {
         debugModeComboBox.setSelectedItem(labelNLS.getString("any.false.label", "False"));
      }

      try
      {
         if (cattrs.getLoadAtStartup() == false)
         {
            loadAtStartupComboBox.setSelectedItem(labelNLS.getString("any.false.label", "False"));
         }
         else
         {
            loadAtStartupComboBox.setSelectedItem(labelNLS.getString("any.true.label", "True"));
         }
      }
      catch (AttributeNotSetException anse)
      {
         loadAtStartupComboBox.setSelectedItem(labelNLS.getString("any.true.label", "True"));
      }


      Vector uriStrings= new Vector();
      if ((!(modeBits.get(BasePropertySheet.EDIT_DEFAULTS_MODE)) && ( !modeBits.get(BasePropertySheet.CREATE_MODE))) ||
          (modeBits.get(BasePropertySheet.CREATE_MODE)) && (modeBits.get(BasePropertySheet.MODEL_MODE)))
      {
         try
         {
            uriStrings = cattrs.getURIPaths();
            oldURIs = cattrs.getURIPaths();
         }
         catch (AttributeNotSetException anse)
         {
         }
      }

      DefaultListModel model = (DefaultListModel)uriList.getModel();
      model.clear();
      for (int i=0;i<uriStrings.size();i++)
      {
         model.addElement(uriStrings.elementAt(i));
      }

      if (modeBits.get(BasePropertySheet.CREATE_MODE))
      {
         groupListComboBox.setEditable(false);
      }
      else
      {
         groupListComboBox.setEnabled(false);
         groupListComboBox.setEditable(false);
      }

      if (modeBits.get(EDIT_MODE) || 
          modeBits.get(CONTENT_PANE_MODE))
      {
         try
         {
            String code = cattrs.getCodeActive();
            codeActiveT.setText(code);
         }
         catch (AttributeNotSetException anse)
         {
            codeActiveT.setText(labelNLS.getString("propertySheet.attributeNotSet","Attribute not set"));
         }
         try
         {
            Hashtable h = cattrs.getInitParamsActive();
            initParmsActivePE.setValues(h);
         }
         catch (AttributeNotSetException anse)
         {
         }
         try
         {
            int debugMode = cattrs.getDebugMode();
            if (debugMode>0)
            {
               debugModeActiveT.setText(labelNLS.getString("any.true.label", "True"));
            }
            else
            {
               debugModeActiveT.setText(labelNLS.getString("any.false.label", "False"));
            }
         }
         catch (AttributeNotSetException anse)
         {
            debugModeActiveT.setText(labelNLS.getString("any.false.label", "False"));
         }
         try
         {
            boolean las = cattrs.getLoadAtStartupActive();
            if (las)
            {
               loadAtStartupActiveT.setText(labelNLS.getString("any.true.label", "True"));
            }
            else
            {
               loadAtStartupActiveT.setText(labelNLS.getString("any.false.label", "False"));
            }
         }
         catch (AttributeNotSetException anse)
         {
            loadAtStartupActiveT.setText(labelNLS.getString("any.false.label", "False"));
         }
         try
         {
            Vector uris = cattrs.getURIPathsActive();
            DefaultListModel defmodel = (DefaultListModel)uriListActive.getModel();
            defmodel.clear();
            for (int i=0;i<uris.size();i++)
            {
               defmodel.addElement(uris.elementAt(i));
            }
         }
         catch (AttributeNotSetException anse)
         {
         }

      }
      if ( modeBits.get(BasePropertySheet.EDIT_DEFAULTS_MODE))
      {
         nameTextField.setEnabled(false);
         groupListComboBox.setEnabled(false);
         codeT.setEnabled(false);
         addButton.setEnabled(false);
         editButton.setEnabled(false);
         deleteButton.setEnabled(false);
         uriList.setEnabled(false);
      }

      if ( modeBits.get(BasePropertySheet.READ_ONLY_MODE))
      {
         descriptionT.setEditable(false);
         codeT.setEditable(false);

         loadAtStartupComboBox.setEditable(false);
         loadAtStartupComboBox.setEnabled(false);

         debugModeComboBox.setEditable(false);
         debugModeComboBox.setEnabled(false);
         addButton.setEnabled(false);
         editButton.setEnabled(false);
         deleteButton.setEnabled(false);
         uriList.setEnabled(false);


         initParmsPE.setEnabled(false);
      }

      if (modeBits.get(BasePropertySheet.CREATE_MODE)&& !modeBits.get(BasePropertySheet.MODEL_MODE))
      {
         // App Server Choices
         ro = Utils.getAllObjectsOfType("com.ibm.ejs.sm.beans.ServletGroup");
         if (ro == null || ro.length == 0)
         {
            //          String errorString = labelNLS.getString("noinstancesfound.error"," No {1} instances found to create {2} under");
            showError(labelNLS.getString("servlet.nogroupsfound","No Web Applications found to create Servlet under"));
            Tr.warning(tc, "No.Web.Applications.found.to.create.Servlet.under");
            return;
         }
         for (int i = 0 ;i < ro.length; i++)
         {
            groupListComboBox.addItem(ro[i]);
         }

         RepositoryObjectProxy rop = SelectionCoordinator.getSelectedProxy();
         if (rop instanceof TypeInstance)
         {
            groupListComboBox.setSelectedItem(rop);
         }
         else
         {
            groupListComboBox.setSelectedItem(ro[0]);
         }
      }
      else if (!modeBits.get(PropertySheet.STAND_ALONE_MODE))
      {
         if (SelectionCoordinator.getSelectedProxy().getParent() != null)
         {
            RepositoryObjectProxy sproxy = SelectionCoordinator.getSelectedProxy().getParent();
            groupListComboBox.addItem(sproxy);
            groupListComboBox.setSelectedItem(sproxy);
         }
         else
         {
            RepositoryObjectProxy tempproxy = SelectionCoordinator.getSelectedProxy();
            groupListComboBox.addItem (tempproxy);
            groupListComboBox.setSelectedItem (tempproxy);
         }
      }

      Tr.exit(tc, "reflectAttributes");
      return;
   }

   private void adjustSize(JComboBox c)
   {
      Dimension d = c.getSize();
      d.setSize(FIELD_SIZE, d.height);
      c.setSize(d);
   }

   public void actionPerformed(ActionEvent e)
   {
      Tr.entry(tc, "actionPerformed",e);
      String command = e.getActionCommand();
      if (command.equals("Delete"))
      {
         /*
         ** All this just because AbstractListModel doesn't have a remove
         ** and DefaultListModel doesn't have a way to set a Vector of objects
         ** either way we have to recreate a vector or array of objects
         */
         DefaultListModel model = (DefaultListModel)uriList.getModel();
         int index = uriList.getSelectedIndex();
         if (index >= 0 )
         {
            model.remove(index);
            dirtyURIs = true;
         }
      }
      else if (command.equals("Add"))
      {
         if (groupListComboBox.getSelectedIndex()<0)
         {
            EJSConsoleDialog.showInfo(labelNLS.getString("servlet.group.pleaseselectwebapp", "Please Select, Then Edit a Unique Web Path for this Servlet"));
         }
         else
         {
            new SRTreePanel("add");
         }
      }
      else if (command.equals("Edit"))
      {
         if (uriList.getSelectedIndex() >=0)
         {
            if (groupListComboBox.getSelectedIndex()<0)
            {
               EJSConsoleDialog.showInfo(labelNLS.getString("servlet.group.pleaseselectwebapp", "Please Select, Then Edit a Unique Web App for this Servlet"));
            }
            else
            {
               new SRTreePanel(uriList.getSelectedIndex());
            }
         }
         else
         {
            EJSConsoleDialog.showInfo(labelNLS.getString("servlet.uri.pleaseselecturi", "Please Select, Then Edit a unique Web Path"));
         }

      }
      Tr.exit(tc, "actionPerformed");
   }

   private ListModel getModel ()
   {
      if (uriList != null)
      {
         return uriList.getModel();
      }
      else
      {
         return null;
      }
   }


   class SRTreePanel implements ActionListener
   {

      private JLabel  uriL;
      private JTextField uriT;

      private JButton okButton;
      private JButton cancelButton;
      private JPanel resourceButtonPanel;

      private JScrollPane typeTreeScroller;
      private JPanel scrollerPanel;
      private JPanel attachToPanel;
      private ISubTreeAdapter adapter;
      private JPanel resourcePanel;

      private JLabel virtualHostL;
      private JLabel virtualHostT;

      private JLabel sguriT;
      private String virtualHost;
      private String sguri;

      protected boolean doAdd = false;

      private int updateIndex;
      private RepositoryObjectProxy sgProxy = (RepositoryObjectProxy)getGroupSelection() ;

      public SRTreePanel()
      {
         new SRTreePanel(-1);
      }

      public SRTreePanel (String action)
      {
         if (action.equals ("add"))
         {
            doAdd = true;
         }
         updateIndex = -1;
         popupResourceTreePanel();
      }

      public SRTreePanel(int index)
      {
         updateIndex = index;
         popupResourceTreePanel();
      }

      public void popupResourceTreePanel()
      {

         Tr.entry(tc, "popupResourceTreePanel");
         System.out.println ("popupResourceTreePanel");

         okButton = new JButton()
         {
            public Dimension getPreferredSize()
            {
               return UIConstants.defaultButtonSize;
            }
            public Dimension getMaximumSize()
            {
               return UIConstants.defaultButtonSize;
            }
         };
         okButton.setText(labelNLS.getString("any.done.label","OK"));
         okButton.setActionCommand("Ok");
         okButton.addActionListener(this);

         cancelButton = new JButton()
         {
            public Dimension getPreferredSize()
            {
               return UIConstants.defaultButtonSize;
            }
            public Dimension getMaximumSize()
            {
               return UIConstants.defaultButtonSize;
            }
         };
         cancelButton.setText(labelNLS.getString("any.cancel.label","Cancel"));
         cancelButton.setActionCommand("Cancel");
         cancelButton.addActionListener(this);

         resourceButtonPanel = new JPanel();
         resourceButtonPanel.add(okButton);
         resourceButtonPanel.add(cancelButton);

         uriT = new JTextField(FIELD_SIZE);

         // vh
         virtualHostL = getLabel(labelNLS.getString("servlet.virtualhost.label", "Virtual Host"));
         virtualHostT = new JLabel();

         // uri = sguri + uriT.getText
         uriL = getLabel(labelNLS.getString("servlet.uriname.label", "Servlet Path"));
         sguriT = new JLabel(); 


         Attributes attrs = sgProxy.getAttributes();
         ServletGroupAttributes sgattrs = null;

         // If this is an instanceOf ModelAttributes, then the attributes obtained
         // from the Model are the ServletAttributes. So in this case, we need to
         // derive the ServletGroupAttributes from the Model Servlet.
         if (attrs instanceof ModelAttributes)
         {
            try
            {
            RepositoryObjectProxy servletGroup = sgProxy;
            if (!modeBits.get(CREATE_MODE))
            {
              servletGroup = sgProxy.getParent(); 
            }
            if (servletGroup.getAttributes() instanceof ModelAttributes)
               {
                  if (servletGroup.getAttributes() != null)
                  {
                     ModelAttributes modelAttrs = (ModelAttributes)servletGroup.getAttributes();
                     if (modelAttrs.getAttributes() != null)
                     {
                        sgattrs = (ServletGroupAttributes)modelAttrs.getAttributes();       
                     }
                  }
               }
            }
            catch (AttributeNotSetException anse)
            {
            }
         }
         else if (attrs instanceof ServletGroupAttributes)
         {
            sgattrs = (ServletGroupAttributes) attrs;
         }   
         else if (attrs instanceof ServletAttributes)
         {
            RepositoryObjectProxy servletGroup = sgProxy.getParent();
            sgattrs = (ServletGroupAttributes)servletGroup.getAttributes();
         }

         if (sgattrs != null)
         {
            try
            {
               String uri = sgattrs.getURIPath();
               int slashIndex = uri.indexOf("/");

               if (slashIndex >=0)
               {
                  virtualHost = uri.substring(0,slashIndex);
                  sguri = uri.substring(slashIndex,uri.length());
                  sguriT.setText(sguri);
                  if (!sguri.endsWith("/"))
                  {
                     uriT.setText("/");
                  }
                  virtualHostT.setText(virtualHost);
                  if (updateIndex > -1)
                  {
                     String uriString = uriList.getSelectedValue().toString();
                     int uriIndex = uri.length();
                     if ( uriIndex <= uriString.length())
                     {
                        uriT.setText(uriString.substring(uriIndex,uriString.length()));
                     }
                  }
               }
            }
            catch (AttributeNotSetException anse)
            {
               showError(labelNLS.getString("servlet.nowebappuri","No Web Path found for Web Application"));
               return;
            }
         }
         else
         {
            sguri = "";
            sguriT.setText(sguri);
            uriT.setText("");
            virtualHost = "";
            virtualHostT.setText (virtualHost);
            if ((uriList.getSelectedValue() != null) && !doAdd)
               uriT.setText(uriList.getSelectedValue().toString());
         }

         GridBagConstraints attachPanelgbc = new GridBagConstraints();
         GridBagLayout attachPanelgbl = new GridBagLayout();
         attachPanelgbc.fill = GridBagConstraints.HORIZONTAL;
         attachPanelgbc.anchor = GridBagConstraints.NORTH;

         attachPanelgbc.insets = new Insets(4,4,4,4);
         attachToPanel = new JPanel();
         attachToPanel.setLayout(attachPanelgbl);

         attachPanelgbc.gridx=0;attachPanelgbc.gridy=0;
         attachPanelgbc.weightx=0;
         attachToPanel.add(virtualHostL,attachPanelgbc);
         attachPanelgbc.gridx=1;
         attachPanelgbc.weightx=1;
         attachToPanel.add(virtualHostT,attachPanelgbc);

         attachPanelgbc.weightx=0;
         attachPanelgbc.gridx=0;attachPanelgbc.gridy=1;;
         attachToPanel.add(uriL,attachPanelgbc);

         JPanel panel = new JPanel();
         panel.setLayout(new BorderLayout());
         //panel.setBorder(uriT.getBorder());
         panel.setBorder(new CompoundBorder(BasicFieldBorder.getFieldBorder(),BasicMarginBorder.getMarginBorder()));
         sguriT.setBorder(BorderFactory.createEmptyBorder());
         uriT.setBorder(BorderFactory.createEmptyBorder());
         panel.add(sguriT,BorderLayout.WEST);
         panel.add(uriT,BorderLayout.CENTER);
         attachPanelgbc.weightx=1;
         attachPanelgbc.gridx=1;attachPanelgbc.gridy=1;;
         attachToPanel.add(panel,attachPanelgbc);

         attachToPanel.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(),labelNLS.getString("servlet.attachto","Edit Servlet Web Path")));

         resourcePanel = new JPanel();
         resourcePanel.setLayout(new BorderLayout());
         resourcePanel.add(attachToPanel,BorderLayout.CENTER);
         resourcePanel.add(resourceButtonPanel,BorderLayout.SOUTH);
         resourcePanel.setPreferredSize( new Dimension(400,150));

         ComponentManager.displaySubFrame(resourcePanel,
                                          labelNLS.getString("servlet.addURI", "Add Web Path to Servlet"),
                                          true,  // modal
                                          false, // needs scroller
                                          true); // allowed to exceed max frame dimensions
         Tr.exit(tc, "popupResourceTreePanel");

      }

      public void actionPerformed(ActionEvent e)
      {
         Tr.entry(tc, "actionPerformed",e);
         String command = e.getActionCommand();
         if (command.equals("Ok"))
         {
            DefaultListModel model = (DefaultListModel)uriList.getModel();
            if (model != null)
            {
               String uri = uriT.getText();
               if (!uri.startsWith("/"))
               {
                  //uri="/"+uri;
               }
               String updateURI = virtualHostT.getText()+sguri+uri;
               updateURI = removeDoubleSlash(updateURI);
               if (updateIndex >= 0)
               {
                  model.setElementAt(updateURI,updateIndex);
               }
               else
               {
                  model.addElement(updateURI);
               }
               dirtyURIs = true;
            }
            ComponentManager.clearSubFrame(resourcePanel);
         }
         else if (command.equals("Cancel"))
         {
            ComponentManager.clearSubFrame(resourcePanel);
         }
         Tr.entry(tc, "actionPerformed");
      }
   }

   public static String removeDoubleSlash(String src) 
   {
      if (tc.isEntryEnabled()) Tr.entry(tc, "removeDoubleSlash");

      String dest = new String("");
      char c = 'a';
      char lc = 'a';

      for (int i = 0; i < src.length(); i++)
      {
         c = src.charAt(i);
         if (c == '/')
         {
            if (lc != '/')
            {
               dest += c;
            }
         }
         else
         {
            dest += c;
         }

         lc = c;
      }

      if (tc.isEntryEnabled()) Tr.exit(tc, "removeDoubleSlash");
      return dest;
   }
}

